import random
import bz2
import numpy
from numpy import array, where, concatenate
from numpy import kron, ones, sqrt, sum
from os.path import exists
from esvm.mldata import convert
try:
    import arff
    have_arff = True
except ImportError:
    have_arff = False



def create_dataset():
    """Read the file with first 100k sequences from C. elegans
    and generate some easier datasets.
    """

    if not have_arff:
        print 'import arff failed, currently cannot create data'
        return

    # convert data to arff format
    gen_arff('C_elegans_acc_100000.fasta.bz2','C_elegans_acc_gc.arff','C_elegans_acc_seq.arff',\
             'C_elegans_acc_seq2.arff','C_elegans_acc_freq.arff',\
             num_seqs=100000,subset=True,overwrite=True,normalise=False,\
             max_pos=200,max_neg=2000)

    print 'Convert from arff to csv and fasta'
    convert('C_elegans_acc_gc.arff','C_elegans_acc_gc.csv','vec')
    convert('C_elegans_acc_seq.arff','C_elegans_acc_seq.csv','seq')
    convert('C_elegans_acc_freq.arff','C_elegans_acc_freq.csv','vec')
    convert('C_elegans_acc_seq2.arff','C_elegans_acc_seq2.csv','mseq')
    convert('C_elegans_acc_seq.arff','C_elegans_acc_seq.fa','seq')


def create_modsel():
    """Read the file with last 100k sequences from C. elegans
    and generate some easier datasets.
    """

    if not have_arff:
        print 'import arff failed, currently cannot create data'
        return

    # convert data to arff format
    gen_arff('C_elegans_acc_modsel.fasta.bz2','C_elegans_acc_modsel_gc.arff','C_elegans_acc_modsel_seq.arff',\
             'C_elegans_acc_modsel_seq2.arff','C_elegans_acc_modsel_freq.arff',\
             num_seqs=100000,subset=True,overwrite=True,normalise=False,\
             max_pos=200,max_neg=2000)

    print 'Convert from arff to csv and fasta'
    convert('C_elegans_acc_modsel_gc.arff','C_elegans_acc_modsel_gc.csv','vec')
    convert('C_elegans_acc_modsel_seq.arff','C_elegans_acc_modsel_seq.csv','seq')
    convert('C_elegans_acc_modsel_freq.arff','C_elegans_acc_modsel_freq.csv','vec')
    convert('C_elegans_acc_modsel_seq2.arff','C_elegans_acc_modsel_seq2.csv','mseq')
    convert('C_elegans_acc_modsel_seq.arff','C_elegans_acc_modsel_seq.fa','seq')


def gen_arff(fastafilename,gcfilename,seqfilename,seq2filename,specfilename,\
             num_seqs=100000,subset=False,max_pos=200,max_neg=2000,\
             overwrite=False,normalise=True):
    """If data not yet created, generate 2 arff files
    - containing the two dimensional GC content before and after splice site
    - containing the sequence around the splice site.
    """
    if (exists(gcfilename) and exists(seqfilename)) and not overwrite:
        return

    print 'Creating %s and %s from %s' % (gcfilename,seqfilename,fastafilename)

    if fastafilename.find('acc')!= -1:
        # acceptor, AG at [40:42]
        window = (-40, 197, 42)
    elif fastafilename.find('don')!= -1:
        # donor, GT or GC at [40:42]
        window = (-40, 200, 42)
    else:
        print "Error: Cannot determine whether donor or acceptor"

    [strings, lab]=read_data(bz2.BZ2File(fastafilename), num_seqs, window)
    # Only a subset of the examples are used.
    if subset:
        [strings, lab] = take_subset(strings, lab, max_pos, max_neg)

    gcs=count_gs_and_cs(strings, (0, -window[0]), (-window[0]+2, -window[0]+2+window[2]))

    seq_upstream = []
    seq_downstream = []
    for curstr in strings:
        seq_upstream.append(curstr[0:-window[0]])
        seq_downstream.append(curstr[(-window[0]+2):(-window[0]+2+window[2])])
    seq_upstream = array(seq_upstream)
    seq_downstream = array(seq_downstream)

    spec_up = count_nt_freq(seq_upstream)
    spec_down = count_nt_freq(seq_downstream)

    if normalise:
        gcs = normalise_features(gcs)
        spec_up = normalise_features(spec_up)
        spec_down = normalise_features(spec_down)

    # sequence file
    alist = [('label',1,[]),('sequence',0,[])]
    f = open(seqfilename,'w')
    arff.arffwrite(f,alist,zip(lab,strings),name=fastafilename,comment='Converted from '+fastafilename)
    f.close()

    # 2 sequence file
    alist = [('label',1,[]),('upstream sequence',0,[]),('downstream sequence',0,[])]
    f = open(seq2filename,'w')
    arff.arffwrite(f,alist,zip(lab,seq_upstream,seq_downstream),\
                   name=fastafilename,comment='Converted from '+fastafilename)
    f.close()

    # gc contents
    alist = [('label',1,[]),('upstream',1,[]),('downstream',1,[])]
    data = []
    for ix,curlab in enumerate(lab):
        data.append((curlab,gcs[0,ix],gcs[1,ix]))
    f = open(gcfilename,'w')
    arff.arffwrite(f,alist,data,name=fastafilename,comment='Converted from '+fastafilename)
    f.close()

    # spectrum
    alist = [('label',1,[]),\
             ('upA',1,[]),('upC',1,[]),('upG',1,[]),('upT',1,[]),\
             ('downA',1,[]),('downC',1,[]),('downG',1,[]),('downT',1,[])]
    data = []
    for ix,curlab in enumerate(lab):
        data.append((curlab,spec_up[0,ix],spec_up[1,ix],spec_up[2,ix],spec_up[3,ix],\
                     spec_down[0,ix],spec_down[1,ix],spec_down[2,ix],spec_down[3,ix]))
    if len(specfilename)>0:
        f = open(specfilename,'w')
        arff.arffwrite(f,alist,data,name=fastafilename,comment='Converted from '+fastafilename)
        f.close()


def take_subset(strings, lab, max_pos=200, max_neg=2000):
    """Take a subset of the classes to the maximum numbers determined by
    max_pos and max_neg
    """
    random.seed(123456789)

    pos_idx = where(lab>0)[0]
    neg_idx = where(lab<0)[0]
    num_pos = len(pos_idx)
    num_neg = len(neg_idx)

    assert(num_pos < num_neg)
    assert(max_pos < max_neg)

    max_pos = min(max_pos,num_pos)
    max_neg = min(max_neg,num_neg)

    neg_sub_idx = array(random.sample(neg_idx,max_neg))
    assert(all(lab[neg_sub_idx]<0))
    pos_sub_idx = array(random.sample(pos_idx,max_pos))
    assert(all(lab[pos_sub_idx]>0))

    strings = concatenate((strings[pos_sub_idx],strings[neg_sub_idx]))
    lab = concatenate((lab[pos_sub_idx],lab[neg_sub_idx]))

    return (strings,lab)

def balance_classes(strings, lab, max_examples=1200,ratio=5.0):
    """Take a subset of negative examples such that
    the number of examples in the negative class are limited to ratio.

    Also limit the maximum number of examples.
    """
    random.seed(123456789)

    pos_idx = where(lab>0)[0]
    neg_idx = where(lab<0)[0]
    num_pos = len(pos_idx)
    num_neg = len(neg_idx)
    assert(num_pos < num_neg)

    max_pos = int(float(max_examples)/(ratio+1.0))

    if num_pos < max_pos:
        max_pos = num_pos

    pos_idx = pos_idx[:max_pos]
    num_pos = len(pos_idx)
    max_neg = int(num_pos*ratio)
    if num_neg < max_neg:
        max_neg = num_neg

    sub_idx = array(random.sample(neg_idx,max_neg))
    assert(all(lab[sub_idx]<0))

    strings = concatenate((strings[pos_idx],strings[sub_idx]))
    lab = concatenate((lab[pos_idx],lab[sub_idx]))

    return (strings,lab)

def normalise_features(feats):
    """Normalise each feature to zero mean and unit variance.
    Assume features are column wise matrix.

    """
    (numdim,numex) = feats.shape

    M = sum(feats,axis=1)/numex
    M = M.reshape(numdim,1)

    M2 = sum(feats**2,axis=1)/numex
    M2 = M2.reshape(numdim,1)
    SD = sqrt(M2-M**2)
    onevec = ones((1,numex))
    feats = (feats - kron(onevec,M))/(kron(onevec,SD))

    return feats

def read_data(f, num, window):
    """Read the fasta file containing splice sites."""
    labels=num*[0]
    strings=num*[0]

    l1 = f.readline()
    l2 = f.readline()
    line = 0
    num_alt_consensus = 0
    while l1 and l2 and line<num:
        consensus = l2[:-1][window[1]:window[1]+2]
        if (consensus == 'AG') or (consensus == 'GT'):
            if 'label=-1' in l1:
                labels[line]=-1
            elif 'label=1' in l1:
                labels[line]=+1
            else:
                print "error in line %d" % line
                return

            strings[line] = l2[:-1][window[1]+window[0] : window[1]+window[2]]
            line+=1
        else:
            num_alt_consensus+=1
            if consensus != 'GC':
                print line, consensus

        l1=f.readline()
        l2=f.readline()

    print "Number of GC consensus sites: %d" %num_alt_consensus
    if line+num_alt_consensus!=num:
        print "error reading file"
        return
    else:
        strings = strings[:line+1]
        labels = labels[:line+1]
        return (array(strings), array(labels, dtype=numpy.double))

def count_gs_and_cs(strings, range1, range2):
    """Count the number of G and C in the two ranges."""
    num=len(strings)
    gc_count=num*[(0,0)]

    for i in xrange(num):
        x=float(strings[i].count('G', range1[0], range1[1]) +
                        strings[i].count('C', range1[0], range1[1])) / abs(range1[1]-range1[0])
        y=float(strings[i].count('G', range2[0], range2[1]) +
                        strings[i].count('C', range2[0], range2[1])) / abs(range2[1]-range2[0])
        gc_count[i]=(x,y)

    return array(gc_count).T


def count_nt_freq(strings):
    """Count the nucleotide frequencies"""
    num = len(strings)
    strlen = len(strings[0])
    ntfreq = num*[(0,0,0,0)]

    for ix in xrange(num):
        a=float(strings[ix].count('A')) / strlen
        c=float(strings[ix].count('C')) / strlen
        g=float(strings[ix].count('G')) / strlen
        t=float(strings[ix].count('T')) / strlen
        ntfreq[ix]=(a,c,g,t)

    return array(ntfreq).T




#!/usr/bin/env python
data = '../data/fm_train_real.dat'
parameter_list = [[data]]

def converter_isomap_modular (data_fname):
	try:
		from modshogun import RealFeatures, Isomap, CSVFile

		features = RealFeatures(CSVFile(data))

		converter = Isomap()
		converter.set_k(20)
		converter.set_target_dim(1)
		converter.apply(features)

		return features
	except ImportError:
		print('No Eigen3 available')

if __name__=='__main__':
	print('Isomap')
	converter_isomap_modular(*parameter_list[0])




#!/usr/bin/env python
from numpy.random import seed
seed(42)

parameter_list=[[7],[8]]

def kernel_custom_modular (dim=7):
	from numpy.random import rand, seed
	from numpy import array, float32
	from modshogun import RealFeatures
	from modshogun import CustomKernel

	seed(17)
	data=rand(dim, dim)
	feats=RealFeatures(data)
	symdata=data+data.T
	lowertriangle=array([symdata[(x,y)] for x in range(symdata.shape[1])
		for y in range(symdata.shape[0]) if y<=x])

	kernel=CustomKernel()

	# once with float64's
	kernel.set_triangle_kernel_matrix_from_triangle(lowertriangle)
	km_triangletriangle=kernel.get_kernel_matrix()

	kernel.set_triangle_kernel_matrix_from_full(symdata)
	km_fulltriangle=kernel.get_kernel_matrix()

	kernel.set_full_kernel_matrix_from_full(symdata)
	km_fullfull=kernel.get_kernel_matrix()

	# now once with float32's
	data=array(data,dtype=float32)

	kernel.set_triangle_kernel_matrix_from_triangle(lowertriangle)
	km_triangletriangle=kernel.get_kernel_matrix()

	kernel.set_triangle_kernel_matrix_from_full(symdata)
	km_fulltriangle=kernel.get_kernel_matrix()

	kernel.set_full_kernel_matrix_from_full(symdata)
	km_fullfull=kernel.get_kernel_matrix()
	return km_fullfull,kernel

if __name__=='__main__':
	print('Custom')
	kernel_custom_modular(*parameter_list[0])




from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()
traindna=lm.load_dna('../data/fm_train_dna.dat')
cubedna=lm.load_cubes('../data/fm_train_cube.dat')
parameter_list=[[traindna,cubedna,3,0,'n'],[traindna,cubedna,4,0,'n']]

def distribution_histogram(fm_train=traindna,fm_cube=cubedna,order=3,
			    gap=0,reverse='n'):

#	sg('new_distribution', 'HISTOGRAM')
	sg('add_preproc', 'SORTWORDSTRING')

	sg('set_features', 'TRAIN', fm_train, 'DNA')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TRAIN')
#	sg('train_distribution')
#	histo=sg('get_histogram')

#	num_examples=11
#	num_param=sg('get_histogram_num_model_parameters')
#	for i in xrange(num_examples):
#		for j in xrange(num_param):
#			sg('get_log_derivative %d %d' % (j, i))

#	sg('get_log_likelihood')
#	return sg('get_log_likelihood_sample')

if __name__=='__main__':
	print('Histogram')
	distribution_histogram(*parameter_list[0])



#!/usr/bin/env python

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import sys
from distutils.core import setup, Extension


sys.stdout.write('-------------------------------------------\n')
sys.stdout.write('easysvm - A front end to the shogun toolbox\n')
sys.stdout.write('-------------------------------------------\n\n')


try:
    import numpy
except:
    sys.stderr.write("WARNING: did not find 'numpy'\n")

try:
    import pylab
except:
    sys.stderr.write("WARNING: did not find 'pylab'\n")

try:
    import shogun
except:
    sys.stderr.write("WARNING: did not find 'shogun'\n")
    sys.stderr.write("         shogun is a required back end.\n")
    sys.stderr.write("         See shogun website: http://www.shogun-toolbox.org)\n")

try:
    import arff
except:
    sys.stderr.write("WARNING: did not find 'arff'\n")
    sys.stderr.write("         arff is required for reading and writing ARFF data files\n")
    sys.stderr.write("         See arff website: http://www.mit.edu/~sav/arff/\n\n")
    sys.stderr.write("         All other functionality should be ok.\n\n")



setup (name = 'easysvm',
       version = '0.3.3',
       description = 'easysvm - A front end to the shogun toolbox',
       author = ['Cheng Soon Ong', 'Gunnar Raetsch' ],
       author_email = ['chengsoon.ong@tuebingen.mpg.de','gunnar.raetsch@tuebingen.mpg.de'],
       license='GPLv3',
       url = 'http://www.fml.tuebingen.mpg.de/raetsch/projects/easysvm',
       py_modules=['esvm.datafuncs','esvm.mldata','esvm.mldata_arff','esvm.experiment','esvm.parse',\
                   'esvm.plots','esvm.poim','esvm.utils','splicesites.utils'],
       packages=['esvm','splicesites'],
       scripts=['scripts/easysvm.py','scripts/datagen.py'],
       long_description="""
       easysvm is a front end to the shogun toolbox. It aims to be a
       'quick start' tutorial for users interested in shogun. As such,
       it covers only the absolute basics of machine learning.
       More advanced users should directly use the interfaces provided
       by shogun.

       This setup.py installs two modules:
       - esvm (which contains some basic tools for machine learning with SVMs)
       - splicesites (which contains tools for splice site prediction)

       It also installs two scripts (easysvm.py and datagen.py) in the relevant binary directory.
       The two scripts provide a command line interface to the toolkit.
       """
       )




#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt

from modshogun	import RealFeatures
from modshogun	import MulticlassModel, MulticlassSOLabels, RealNumber, DualLibQPBMSOSVM, DirectorStructuredModel
from modshogun	import BMRM, PPBMRM, P3BMRM, ResultSet, RealVector
from modshogun	import StructuredAccuracy

class MulticlassStructuredModel(DirectorStructuredModel):
	def __init__(self,features,labels):
		DirectorStructuredModel.__init__(self)
		self.set_features(features)
		self.set_labels(labels)
		self.dim = features.get_dim_feature_space()*labels.get_num_classes()
		self.n_classes = labels.get_num_classes()
		self.n_feats = features.get_dim_feature_space()
		#self.use_director_risk()
	def get_dim(self):
		return self.dim
	def argmax(self,w,feat_idx,training):
		feature_vector = self.get_features().get_feature_vector(feat_idx)
		label = None
		if training == True:
			label = int(RealNumber.obtain_from_generic(self.get_labels().get_label(feat_idx)).value)
		ypred = 0
		max_score = -1e10
		for c in xrange(self.n_classes):
			score = 0.0
			for i in xrange(self.n_feats):
				score += w[i+self.n_feats*c]*feature_vector[i]
			if training == True:
				score += (c!=label)
			if score > max_score:
				max_score = score
				ypred = c

		res = ResultSet()
		res.score = max_score
		res.psi_pred = RealVector(self.dim)
		res.psi_pred.zero()
		for i in xrange(self.n_feats):
			res.psi_pred[i+self.n_feats*ypred] = feature_vector[i]
		res.argmax = RealNumber(ypred)
		if training == True:
			res.delta = (label!=ypred)
			res.psi_truth = RealVector(self.dim)
			res.psi_truth.zero()
			for i in xrange(self.n_feats):
				res.psi_truth[i+self.n_feats*label] = feature_vector[i]
			for i in xrange(self.n_feats):
				res.score -= w[i+self.n_feats*label]*feature_vector[i]
		return res


def fill_data(cnt, minv, maxv):
	x1 = np.linspace(minv, maxv, cnt)
	a, b = np.meshgrid(x1, x1)
	X = np.array((np.ravel(a), np.ravel(b)))
	y = np.zeros((1, cnt*cnt))
	tmp = cnt*cnt;
	y[0, tmp/3:(tmp/3)*2]=1
	y[0, tmp/3*2:(tmp/3)*3]=2
	return X, y.flatten()

def gen_data():
	covs = np.array([[[0., -1. ], [2.5,  .7]],
		[[3., -1.5], [1.2, .3]],
		[[ 2,  0  ], [ .0,  1.5 ]]])
	X = np.r_[np.dot(np.random.randn(N, dim), covs[0]) + np.array([0, 10]),
			np.dot(np.random.randn(N, dim), covs[1]) + np.array([-10, -10]),
			np.dot(np.random.randn(N, dim), covs[2]) + np.array([10, -10])];
	Y = np.hstack((np.zeros(N), np.ones(N), 2*np.ones(N)))
	return X, Y

def get_so_labels(out):
	N = out.get_num_labels()
	l = np.zeros(N)
	for i in xrange(N):
		l[i] = RealNumber.obtain_from_generic(out.get_label(i)).value
	return l

# Number of classes
M = 3
# Number of samples of each class
N = 10
# Dimension of the data
dim = 2

X, y = gen_data()

cnt = 50

X2, y2 = fill_data(cnt, np.min(X), np.max(X))

labels = MulticlassSOLabels(y)
features = RealFeatures(X.T)

model = MulticlassStructuredModel(features, labels)

lambda_ = 1e1
sosvm = DualLibQPBMSOSVM(model, labels, lambda_)

sosvm.set_cleanAfter(10)	# number of iterations that cutting plane has to be inactive for to be removed
sosvm.set_cleanICP(True)	# enables inactive cutting plane removal feature
sosvm.set_TolRel(0.001)		# set relative tolerance
sosvm.set_verbose(True)		# enables verbosity of the solver
sosvm.set_cp_models(16)		# set number of cutting plane models
sosvm.set_solver(BMRM)		# select training algorithm
#sosvm.set_solver(PPBMRM)
#sosvm.set_solver(P3BMRM)

sosvm.train()

res = sosvm.get_result()
Fps = res.get_hist_Fp_vector()
Fds = res.get_hist_Fd_vector()
wdists = res.get_hist_wdist_vector()

plt.figure()
plt.subplot(221)
plt.title('Fp and Fd history')
plt.plot(xrange(res.get_n_iters()), Fps, hold=True)
plt.plot(xrange(res.get_n_iters()), Fds, hold=True)
plt.subplot(222)
plt.title('w dist history')
plt.plot(xrange(res.get_n_iters()), wdists)

# Evaluation
out = sosvm.apply()

Evaluation = StructuredAccuracy()
acc = Evaluation.evaluate(out, labels)

print "Correct classification rate: %0.4f%%" % ( 100.0*acc )

# show figure
Z = get_so_labels(sosvm.apply(RealFeatures(X2)))
x = (X2[0,:]).reshape(cnt, cnt)
y = (X2[1,:]).reshape(cnt, cnt)
z = Z.reshape(cnt, cnt)

plt.subplot(223)
plt.pcolor(x, y, z, shading='interp')
plt.contour(x, y, z, linewidths=1, colors='black', hold=True)
plt.plot(X[:,0], X[:,1], 'yo')
plt.axis('tight')
plt.title('Classification')
plt.show()



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
parameter_list=[[traindna,testdna,10,20],
		[traindna,testdna,11,21]]

def kernel_weighteddegreestring (fm_train_dna=traindna,fm_test_dna=testdna,
		size_cache=10,degree=20):

	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('set_kernel', 'WEIGHTEDDEGREE', 'CHAR', size_cache, degree)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('WeightedDegreeString')
	kernel_weighteddegreestring(*parameter_list[0])



#!/usr/bin/env python

import sys
import os

# has to be ordered tuple
REPLACEMENTS=[
	[' float64_t', ' float'],
	[' float32_t', ' float'],
	[' float128_t', ' float'],
	[' double', ' float'],
	[' int32_t', '  int'],
	[' uint32_t', ' int'],
	[' int64_t', ' int'],
	[' uint64_t', ' int'],
	[' uint8_t', ' int'],
	[' int16_t', ' int'],
	[' uint16_t', ' int'],
	[' char', ' str'],
	['-> CFeatures', '-> Features'],
]

class Scrub:
	def __init__ (self, extension, input):
		self.infile=input+extension
		self.outfile=self.infile+'.new'

	def run (self):
		input=open(self.infile, 'r')
		output=open(self.outfile, 'w')

		if sys.version_info >= (3,):
			for text in input.readlines():
				for pairs in REPLACEMENTS:
					text=text.replace(pairs[0], pairs[1])
				output.write(text)
		else:
			for text in input.xreadlines():
				for pairs in REPLACEMENTS:
					text=text.replace(pairs[0], pairs[1])
				output.write(text)

		input.close()
		output.close()

		os.rename(self.outfile, self.infile)


if __name__=='__main__':
	offset=sys.argv[2].find('_wrap.cxx')
	if sys.argv[1] == 'stop':
		sys.exit(0)
	scrubby=Scrub(sys.argv[1], sys.argv[2][0:offset])
	scrubby.run()



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list = [[traindat,testdat,1.4,10], [traindat,testdat,1.5,10]]

def kernel_chi2_modular (train_fname=traindat,test_fname=testdat,width=1.4, size_cache=10):
	from modshogun import RealFeatures, Chi2Kernel, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	kernel=Chi2Kernel(feats_train, feats_train, width, size_cache)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('Chi2')
	kernel_chi2_modular(*parameter_list[0])



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_twoclass.dat'

parameter_list = [[traindat,testdat,label_traindat,2.1,1,1e-5],[traindat,testdat,label_traindat,2.2,1,1e-5]]

def classifier_libsvm_modular (train_fname=traindat,test_fname=testdat,label_fname=label_traindat,width=2.1,C=1,epsilon=1e-5):
	from modshogun import RealFeatures, BinaryLabels
	from modshogun import GaussianKernel, LibSVM, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))
	labels=BinaryLabels(CSVFile(label_fname))
	kernel=GaussianKernel(feats_train, feats_train, width)

	svm=LibSVM(C, kernel, labels)
	svm.set_epsilon(epsilon)
	svm.train()

	supportvectors = sv_idx=svm.get_support_vectors()
	alphas=svm.get_alphas()
	predictions = svm.apply(feats_test)
	#print predictions.get_labels()
	return predictions, svm, predictions.get_labels()

if __name__=='__main__':
	print('LibSVM')
	classifier_libsvm_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.multiclass_shared import prepare_data
# run with toy data
[traindat, label_traindat, testdat, label_testdat] = prepare_data()

parameter_list = [[traindat,testdat,label_traindat,label_testdat,2.1,1,1e-5],[traindat,testdat,label_traindat,label_testdat,2.2,1,1e-5]]

def classifier_multiclass_ecoc_ovr (fm_train_real=traindat,fm_test_real=testdat,label_train_multiclass=label_traindat,label_test_multiclass=label_testdat,lawidth=2.1,C=1,epsilon=1e-5):
	from modshogun import RealFeatures, MulticlassLabels
	from modshogun import LibLinear, L2R_L2LOSS_SVC, LinearMulticlassMachine
	from modshogun import ECOCStrategy, ECOCOVREncoder, ECOCLLBDecoder, MulticlassOneVsRestStrategy

	feats_train = RealFeatures(fm_train_real)
	feats_test  = RealFeatures(fm_test_real)

	labels = MulticlassLabels(label_train_multiclass)

	classifier = LibLinear(L2R_L2LOSS_SVC)
	classifier.set_epsilon(epsilon)
	classifier.set_bias_enabled(True)

	mc_classifier = LinearMulticlassMachine(MulticlassOneVsRestStrategy(), feats_train, classifier, labels)
	mc_classifier.train()
	label_mc = mc_classifier.apply(feats_test)
	out_mc = label_mc.get_labels()

	ecoc_strategy = ECOCStrategy(ECOCOVREncoder(), ECOCLLBDecoder())
	ecoc_classifier = LinearMulticlassMachine(ecoc_strategy, feats_train, classifier, labels)
	ecoc_classifier.train()
	label_ecoc = ecoc_classifier.apply(feats_test)
	out_ecoc = label_ecoc.get_labels()

	n_diff = (out_mc != out_ecoc).sum()
	#if n_diff == 0:
	#	print("Same results for OvR and ECOCOvR")
	#else:
	#	print("Different results for OvR and ECOCOvR (%d out of %d are different)" % (n_diff, len(out_mc)))

	if label_test_multiclass is not None:
		from modshogun import MulticlassAccuracy
		labels_test = MulticlassLabels(label_test_multiclass)
		evaluator = MulticlassAccuracy()
		acc_mc = evaluator.evaluate(label_mc, labels_test)
		acc_ecoc = evaluator.evaluate(label_ecoc, labels_test)
		#print('Normal OVR Accuracy = %.4f' % acc_mc)
		#print('ECOC OVR Accuracy   = %.4f' % acc_ecoc)

	return out_ecoc, out_mc

if __name__=='__main__':
	print('MulticlassMachine')
	classifier_multiclass_ecoc_ovr(*parameter_list[0])




#!/usr/bin/env python
from numpy import *

parameter_list=[[1, 0.1,20,100,6,10,0.5,1, 1], [2,0.2,20,100,6,10,0.5,1, 2]]

def regression_libsvr_modular (svm_c=1, svr_param=0.1, n=100,n_test=100, \
		x_range=6,x_range_test=10,noise_var=0.5,width=1, seed=1):

	from modshogun import RegressionLabels, RealFeatures
	from modshogun import GaussianKernel
	from modshogun import LibSVR, LIBSVR_NU_SVR, LIBSVR_EPSILON_SVR

	# reproducable results
	random.seed(seed)

	# easy regression data: one dimensional noisy sine wave
	n=15
	n_test=100
	x_range_test=10
	noise_var=0.5;
	X=random.rand(1,n)*x_range

	X_test=array([[float(i)/n_test*x_range_test for i in range(n_test)]])
	Y_test=sin(X_test)
	Y=sin(X)+random.randn(n)*noise_var

	# shogun representation
	labels=RegressionLabels(Y[0])
	feats_train=RealFeatures(X)
	feats_test=RealFeatures(X_test)

	kernel=GaussianKernel(feats_train, feats_train, width)

	# two svr models: epsilon and nu
	svr_epsilon=LibSVR(svm_c, svr_param, kernel, labels, LIBSVR_EPSILON_SVR)
	svr_epsilon.train()
	svr_nu=LibSVR(svm_c, svr_param, kernel, labels, LIBSVR_NU_SVR)
	svr_nu.train()

	# predictions
	kernel.init(feats_train, feats_test)
	out1_epsilon=svr_epsilon.apply().get_labels()
	out2_epsilon=svr_epsilon.apply(feats_test).get_labels()
	out1_nu=svr_epsilon.apply().get_labels()
	out2_nu=svr_epsilon.apply(feats_test).get_labels()

	return out1_epsilon,out2_epsilon,out1_nu,out2_nu ,kernel

if __name__=='__main__':
	print('LibSVR')
	regression_libsvr_modular(*parameter_list[0])



#!/usr/bin/env python
###########################################################################
# kernel ridge regression
###########################################################################
from numpy import array
from numpy.random import seed, rand
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')
label_traindat = lm.load_labels('../data/label_train_twoclass.dat')


parameter_list = [[traindat,testdat,label_traindat]]

def regression_least_squares_modular (fm_train=traindat,fm_test=testdat,label_train=label_traindat,tau=1e-6):

	from modshogun import RegressionLabels, RealFeatures
	from modshogun import GaussianKernel
	from modshogun import LeastSquaresRegression

	ls=LeastSquaresRegression(RealFeatures(traindat), RegressionLabels(label_train))
	ls.train()
	out = ls.apply(RealFeatures(fm_test)).get_labels()
	return out,ls

if __name__=='__main__':
	print('LeastSquaresRegression')
	regression_least_squares_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat],[traindat,testdat]]

def kernel_weighted_comm_word_string_modular (fm_train_dna=traindat,fm_test_dna=testdat,order=3,gap=0,reverse=True ):
	from modshogun import WeightedCommWordStringKernel
	from modshogun import StringWordFeatures, StringCharFeatures, DNA
	from modshogun import SortWordString

	charfeat=StringCharFeatures(fm_train_dna, DNA)
	feats_train=StringWordFeatures(charfeat.get_alphabet())
	feats_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train)
	feats_train.add_preprocessor(preproc)
	feats_train.apply_preprocessor()

	charfeat=StringCharFeatures(fm_test_dna, DNA)
	feats_test=StringWordFeatures(charfeat.get_alphabet())
	feats_test.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats_test.add_preprocessor(preproc)
	feats_test.apply_preprocessor()

	use_sign=False
	kernel=WeightedCommWordStringKernel(feats_train, feats_train, use_sign)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('WeightedCommWordString')
	kernel_weighted_comm_word_string_modular(*parameter_list[0])



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'

parameter_list = [[traindat,3],[traindat,4]]

def clustering_kmeans_modular (fm_train=traindat,k=3):
	from modshogun import EuclideanDistance, RealFeatures, KMeans, Math_init_random, CSVFile
	Math_init_random(17)

	feats_train=RealFeatures(CSVFile(fm_train))
	distance=EuclideanDistance(feats_train, feats_train)

	kmeans=KMeans(k, distance)
	kmeans.train()

	out_centers = kmeans.get_cluster_centers()
	kmeans.get_radiuses()

	return out_centers, kmeans

if __name__=='__main__':
	print('KMeans')
	clustering_kmeans_modular(*parameter_list[0])




from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()


traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
train_label=lm.load_labels('../data/label_train_multiclass.dat')
parameter_list=[[traindat,testdat, train_label,10,2.1,10.,1e-5,False],
		[traindat,testdat,train_label,10,2.1,11.,1e-4,False]]

def classifier_libsvm_multiclass (fm_train_real=traindat,fm_test_real=testdat,
			label_train_multiclass=train_label,
			size_cache=10, width=2.1,C=10.,
			epsilon=1e-5,use_bias=False):

	sg('set_features', 'TRAIN', fm_train_real)
	sg('set_kernel', 'GAUSSIAN', 'REAL', size_cache, width)

	sg('set_labels', 'TRAIN', label_train_multiclass)
	sg('new_classifier', 'LIBSVM_MULTICLASS')
	sg('svm_epsilon', epsilon)
	sg('c', C)
	sg('svm_use_bias', use_bias)
	sg('train_classifier')

	sg('set_features', 'TEST', fm_test_real)
	result=sg('classify')
	kernel_matrix = sg('get_kernel_matrix', 'TEST')
	return result, kernel_matrix

if __name__=='__main__':
	print('LibSVMMulticlass')
	classifier_libsvm_multiclass(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,3],[traindat,testdat,20]]

def kernel_weighted_degree_string_modular (fm_train_dna=traindat,fm_test_dna=testdat,degree=20):
	from modshogun import StringCharFeatures, DNA
	from modshogun import WeightedDegreeStringKernel, MSG_DEBUG

	feats_train=StringCharFeatures(fm_train_dna, DNA)
	#feats_train.io.set_loglevel(MSG_DEBUG)
	feats_test=StringCharFeatures(fm_test_dna, DNA)

	kernel=WeightedDegreeStringKernel(feats_train, feats_train, degree)

	from numpy import arange,double
	weights=arange(1,degree+1,dtype=double)[::-1]/ \
		sum(arange(1,degree+1,dtype=double))
	kernel.set_wd_weights(weights)
	#from numpy import ones,float64,int32
	#kernel.set_position_weights(ones(len(fm_train_dna[0]), dtype=float64))

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()

    #this is how to serializate the kernel
	#import pickle
	#pickle.dump(kernel, file('kernel_obj.dump','w'), protocol=2)
	#k=pickle.load(file('kernel_obj.dump','r'))


	return km_train, km_test, kernel


if __name__=='__main__':
	print('WeightedDegreeString')
	kernel_weighted_degree_string_modular(*parameter_list[0])



#!/usr/bin/env python
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2012 Heiko Strathmann
# Copyright (C) 2012 Berlin Institute of Technology and Max-Planck-Society
#

from numpy import array
from numpy.random import seed, rand
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')
label_traindat = lm.load_labels('../data/label_train_twoclass.dat')


parameter_list = [[traindat,testdat,label_traindat,2.1,1,1e-5,1e-2], \
                 [traindat,testdat,label_traindat,2.1,1,1e-5,1e-2]]

def modelselection_grid_search_libsvr_modular (fm_train=traindat,fm_test=testdat,label_train=label_traindat,\
				       width=2.1,C=1,epsilon=1e-5,tube_epsilon=1e-2):
    from modshogun import CrossValidation, CrossValidationResult
    from modshogun import MeanSquaredError
    from modshogun import CrossValidationSplitting
    from modshogun import RegressionLabels
    from modshogun import RealFeatures
    from modshogun import GaussianKernel
    from modshogun import LibSVR
    from modshogun import GridSearchModelSelection
    from modshogun import ModelSelectionParameters, R_EXP
    from modshogun import ParameterCombination

    # training data
    features_train=RealFeatures(traindat)
    labels=RegressionLabels(label_traindat)

    # kernel
    kernel=GaussianKernel(features_train, features_train, width)

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #kernel.print_modsel_params()

    labels=RegressionLabels(label_train)

    # predictor
    predictor=LibSVR(C, tube_epsilon, kernel, labels)
    predictor.set_epsilon(epsilon)

    # splitting strategy for 5 fold cross-validation (for classification its better
    # to use "StratifiedCrossValidation", but the standard
    # "StratifiedCrossValidationSplitting" is also available
    splitting_strategy=CrossValidationSplitting(labels, 5)

    # evaluation method
    evaluation_criterium=MeanSquaredError()

    # cross-validation instance
    cross_validation=CrossValidation(predictor, features_train, labels,
	    splitting_strategy, evaluation_criterium)

#	 (optional) repeat x-val (set larger to get better estimates, at least two
    # for confidence intervals)
    cross_validation.set_num_runs(2)

    # (optional) request 95% confidence intervals for results (not actually
    # needed for this toy example)
    cross_validation.set_conf_int_alpha(0.05)

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #predictor.print_modsel_params()

    # build parameter tree to select C1 and C2
    param_tree_root=ModelSelectionParameters()
    c1=ModelSelectionParameters("C1");
    param_tree_root.append_child(c1)
    c1.build_values(-1.0, 0.0, R_EXP);

    c2=ModelSelectionParameters("C2");
    param_tree_root.append_child(c2);
    c2.build_values(-1.0, 0.0, R_EXP);

    # model selection instance
    model_selection=GridSearchModelSelection(cross_validation, param_tree_root)

    # perform model selection with selected methods
    #print "performing model selection of"
    #print "parameter tree"
    #param_tree_root.print_tree()

    #print "starting model selection"
    # print the current parameter combination, if no parameter nothing is printed
    print_state=False
    # lock data before since model selection will not change the kernel matrix
    # (use with care) This avoids that the kernel matrix is recomputed in every
    # iteration of the model search
    predictor.data_lock(labels, features_train)
    best_parameters=model_selection.select_model(print_state)

    # print best parameters
    #print "best parameters:"
    #best_parameters.print_tree()

    # apply them and print result
    best_parameters.apply_to_machine(predictor)
    result=cross_validation.evaluate()
    #print "mean:", result.mean
    #if result.has_conf_int:
    #    print "[", result.conf_int_low, ",", result.conf_int_up, "] with alpha=", result.conf_int_alpha

if __name__=='__main__':
	print('ModelselectionGridSearchLibSVR')
	modelselection_grid_search_libsvr_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()
traindna=lm.load_dna('../data/fm_train_dna.dat')
cubedna=lm.load_cubes('../data/fm_train_cube.dat')
parameter_list=[[traindna,cubedna,3,6,1,list(),list()],
		[traindna,cubedna,3,6,1,list(),list()]]

def distribution_hmm(fm_train=traindna,fm_cube=cubedna,N=3,M=6,
			   order=1,hmms=list(),links=list()):
	sg('new_hmm',N, M)
	sg('set_features', 'TRAIN', fm_cube, 'CUBE')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'WORD', order)
	sg('bw')
	hmm=sg('get_hmm')

	sg('new_hmm', N, M)
	sg('set_hmm', hmm[0], hmm[1], hmm[2], hmm[3])
	likelihood=sg('hmm_likelihood')
	return likelihood

if __name__=='__main__':
	print('HMM')
	distribution_hmm(*parameter_list[0])




#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
parameter_list = [[traindat,testdat,2,10], [traindat,testdat,5,10]]

def kernel_anova_modular (train_fname=traindat,test_fname=testdat,cardinality=2, size_cache=10):
	from modshogun import ANOVAKernel,RealFeatures,CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	kernel=ANOVAKernel(feats_train, feats_train, cardinality, size_cache)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train, km_test, kernel

if __name__=='__main__':
	print('ANOVA')
	kernel_anova_modular(*parameter_list[0])



#!/usr/bin/env python

from numpy import random
from shogun.Classifier import LibSVM
from shogun.Features import RealFeatures, Labels
from shogun.Kernel import LinearKernel


num_feats=23
num_vec=42

scale=2.1
size_cache=10

C=0.017
epsilon=1e-5
tube_epsilon=1e-2
svm=LibSVM()
svm.set_C(C, C)
svm.set_epsilon(epsilon)
svm.set_tube_epsilon(tube_epsilon)

for i in xrange(3):
	data_train=random.rand(num_feats, num_vec)
	data_test=random.rand(num_feats, num_vec)
	feats_train=RealFeatures(data_train)
	feats_test=RealFeatures(data_test)
	labels=Labels(random.rand(num_vec).round()*2-1)

	svm.set_kernel(LinearKernel(size_cache, scale))
	svm.set_labels(labels)

	kernel=svm.get_kernel()
	print "kernel cache size: %s" % (kernel.get_cache_size())

	kernel.init(feats_test, feats_test)
	svm.train()

	kernel.init(feats_train, feats_test)
	print svm.classify().get_labels()

	#kernel.remove_lhs_and_rhs()

	#import pdb
	#pdb.set_trace()




#!/usr/bin/env python
#
# This program is free software you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation either version 3 of the License, or
# (at your option) any later version.
#
# Written (C) 2012-2013 Heiko Strathmann
#
from numpy import *
#from pylab import *

parameter_list = [[1000,10,5,3,pi/4, "opt"], [1000,10,5,3,pi/4, "l2"]]


def statistics_mmd_kernel_selection_combined(m,distance,stretch,num_blobs,angle,selection_method):
	from modshogun import RealFeatures
	from modshogun import GaussianBlobsDataGenerator
	from modshogun import GaussianKernel, CombinedKernel
	from modshogun import LinearTimeMMD
	from modshogun import MMDKernelSelectionCombMaxL2
	from modshogun import MMDKernelSelectionCombOpt
	from modshogun import BOOTSTRAP, MMD1_GAUSSIAN
	from modshogun import EuclideanDistance
	from modshogun import Statistics, Math

	# init seed for reproducability
	Math.init_random(1)

	# note that the linear time statistic is designed for much larger datasets
	# results for this low number will be bad (unstable, type I error wrong)

	# streaming data generator
	gen_p=GaussianBlobsDataGenerator(num_blobs, distance, 1, 0)
	gen_q=GaussianBlobsDataGenerator(num_blobs, distance, stretch, angle)

	# stream some data and plot
	num_plot=1000
	features=gen_p.get_streamed_features(num_plot)
	features=features.create_merged_copy(gen_q.get_streamed_features(num_plot))
	data=features.get_feature_matrix()

	#figure()
	#subplot(2,2,1)
	#grid(True)
	#plot(data[0][0:num_plot], data[1][0:num_plot], 'r.', label='$x$')
	#title('$X\sim p$')
	#subplot(2,2,2)
	#grid(True)
	#plot(data[0][num_plot+1:2*num_plot], data[1][num_plot+1:2*num_plot], 'b.', label='$x$', alpha=0.5)
	#title('$Y\sim q$')

	# create combined kernel with Gaussian kernels inside (shoguns Gaussian kernel is
	# different to the standard form, see documentation)
	sigmas=[2**x for x in range(-3,10)]
	widths=[x*x*2 for x in sigmas]
	combined=CombinedKernel()
	for i in range(len(sigmas)):
		combined.append_kernel(GaussianKernel(10, widths[i]))

	# mmd instance using streaming features, blocksize of 10000
	block_size=10000
	mmd=LinearTimeMMD(combined, gen_p, gen_q, m, block_size)

	# kernel selection instance (this can easily replaced by the other methods for selecting
	# combined kernels
	if selection_method=="opt":
		selection=MMDKernelSelectionCombOpt(mmd)
	elif selection_method=="l2":
		selection=MMDKernelSelectionCombMaxL2(mmd)

	# perform kernel selection (kernel is automatically set)
	kernel=selection.select_kernel()
	kernel=CombinedKernel.obtain_from_generic(kernel)
	#print "selected kernel weights:", kernel.get_subkernel_weights()
	#subplot(2,2,3)
	#plot(kernel.get_subkernel_weights())
	#title("Kernel weights")

	# compute tpye I and II error (use many more trials). Type I error is only
	# estimated to check MMD1_GAUSSIAN method for estimating the null
	# distribution. Note that testing has to happen on difference data than
	# kernel selecting, but the linear time mmd does this implicitly
	mmd.set_null_approximation_method(MMD1_GAUSSIAN)

	# number of trials should be larger to compute tight confidence bounds
	num_trials=5;
	alpha=0.05 # test power
	typeIerrors=[0 for x in range(num_trials)]
	typeIIerrors=[0 for x in range(num_trials)]
	for i in range(num_trials):
		# this effectively means that p=q - rejecting is tpye I error
		mmd.set_simulate_h0(True)
		typeIerrors[i]=mmd.perform_test()>alpha
		mmd.set_simulate_h0(False)

		typeIIerrors[i]=mmd.perform_test()>alpha

	#print "type I error:", mean(typeIerrors), ", type II error:", mean(typeIIerrors)

	return kernel,typeIerrors,typeIIerrors

if __name__=='__main__':
	print('MMDKernelSelectionCombined')
	statistics_mmd_kernel_selection_combined(*parameter_list[0])
	#show()



"""Generator for Regression"""

import shogun.Regression as regression
from shogun.Kernel import GaussianKernel

import fileop
import featop
import dataop
import category


def _compute (params, feats, kernel, pout):
	"""
	Compute a regression and gather result data.

	@param params misc parameters for the regression method
	@param feats features of the kernel/regression
	@param kernel kernel
	@param pout previously gathered data from kernel ready to be written to file
	"""

	kernel.parallel.set_num_threads(params['num_threads'])
	kernel.init(feats['train'], feats['train'])
	params['labels'], labels=dataop.get_labels(feats['train'].get_num_vectors())

	try:
		fun=eval('regression.'+params['name'])
	except AttributeError:
		return

	if params['type']=='svm':
		regression=fun(params['C'], params['epsilon'], kernel, labels)
		regression.set_tube_epsilon(params['tube_epsilon'])
	else:
		regression=fun(params['tau'], kernel, labels)
	regression.parallel.set_num_threads(params['num_threads'])

	regression.train()

	if params['type']=='svm':
		params['bias']=regression.get_bias()
		params['alpha_sum']=0
		for item in regression.get_alphas().tolist():
			params['alpha_sum']+=item
		params['sv_sum']=0
		for item in regression.get_support_vectors():
			params['sv_sum']+=item

	kernel.init(feats['train'], feats['test'])
	params['classified']=regression.apply().get_labels()

	output=pout.copy()
	output.update(fileop.get_output(category.REGRESSION, params))
	fileop.write(category.REGRESSION, output)


def _loop (regressions, feats, kernel, pout):
	"""
	Loop through regression computations, only slightly differing in parameters.

	@param regressions names of the regression methods to loop through
	@param feats features of the kernel/regression
	@param kernel kernel
	@param pout previously gathered data from kernel ready to be written to file
	"""

	for r in regressions:
		r['num_threads']=1
		if r['type']=='svm':
			r['C']=.017
			r['epsilon']=1e-5
			r['accuracy']=r['epsilon']*10
			r['tube_epsilon']=1e-2
			_compute(r, feats, kernel, pout)
			r['C']=.23
			_compute(r, feats, kernel, pout)
			r['C']=1.5
			_compute(r, feats, kernel, pout)
			r['C']=30
			_compute(r, feats, kernel, pout)
			r['epsilon']=1e-4
			r['accuracy']=r['epsilon']*10
			_compute(r, feats, kernel, pout)
			r['tube_epsilon']=1e-3
			_compute(r, feats, kernel, pout)
		elif r['type']=='kernelmachine':
			r['tau']=1e-6
			_compute(r, feats, kernel, pout)
			r['tau']=1e-5
			_compute(r, feats, kernel, pout)
		else:
			continue

		# BUG in SVRLight:
		# glibc detected *** /usr/bin/python: free(): invalid next size (fast)
		if r['name']!='SVRLight':
			r['num_threads']=16
			_compute(r, feats, kernel, pout)

##########################################################################
# public
##########################################################################

def run ():
	"""Run generator for all regression methods."""

	regressions=(
		{'name': 'SVRLight', 'type': 'svm', 'accuracy': 1e-6},
		{'name': 'LibSVR', 'type': 'svm', 'accuracy': 1e-6},
		{'name': 'KRR', 'type': 'kernelmachine', 'accuracy': 1e-8},
	)

	params={
		'name': 'Gaussian',
		'args': {'key': ('width',), 'val': (1.5,)},
		'feature_class': 'simple',
		'feature_type': 'Real',
		'data': dataop.get_rand()
	}
	output=fileop.get_output(category.KERNEL, params)
	feats=featop.get_simple('Real', params['data'])
	kernel=GaussianKernel(10, *params['args']['val'])

	_loop(regressions, feats, kernel, output)




#!/usr/bin/env python
from tools.multiclass_shared import prepare_data

[traindat, label_traindat, testdat, label_testdat] = prepare_data(False)

parameter_list = [[traindat,testdat,label_traindat,label_testdat,2.1,1,1e-5],[traindat,testdat,label_traindat,label_testdat,2.2,1,1e-5]]

def classifier_multiclass_ecoc_discriminant (fm_train_real=traindat,fm_test_real=testdat,label_train_multiclass=label_traindat,label_test_multiclass=label_testdat,lawidth=2.1,C=1,epsilon=1e-5):
    from modshogun import RealFeatures, MulticlassLabels
    from modshogun import LibLinear, L2R_L2LOSS_SVC, LinearMulticlassMachine
    from modshogun import ECOCStrategy, ECOCDiscriminantEncoder, ECOCHDDecoder

    feats_train = RealFeatures(fm_train_real)
    feats_test  = RealFeatures(fm_test_real)

    labels = MulticlassLabels(label_train_multiclass)

    classifier = LibLinear(L2R_L2LOSS_SVC)
    classifier.set_epsilon(epsilon)
    classifier.set_bias_enabled(True)

    encoder = ECOCDiscriminantEncoder()
    encoder.set_features(feats_train)
    encoder.set_labels(labels)
    encoder.set_sffs_iterations(50)

    strategy = ECOCStrategy(encoder, ECOCHDDecoder())

    classifier = LinearMulticlassMachine(strategy, feats_train, classifier, labels)
    classifier.train()
    label_pred = classifier.apply(feats_test)
    out = label_pred.get_labels()

    if label_test_multiclass is not None:
        from modshogun import MulticlassAccuracy
        labels_test = MulticlassLabels(label_test_multiclass)
        evaluator = MulticlassAccuracy()
        acc = evaluator.evaluate(label_pred, labels_test)
        print('Accuracy = %.4f' % acc)

    return out

if __name__=='__main__':
    print('MulticlassMachine')
    classifier_multiclass_ecoc_discriminant(*parameter_list[0])




#!/usr/bin/env python
from modshogun import LongIntFeatures
from numpy import array, int64, all

# create dense matrix A
matrix=array([[1,2,3],[4,0,0],[0,0,0],[0,5,0],[0,0,6],[9,9,9]], dtype=int64)

parameter_list = [[matrix]]

# ... of type LongInt
def features_dense_longint_modular (A=matrix):
	a=LongIntFeatures(A)
	# get first feature vector and set it

	a.set_feature_vector(array([1,4,0,0,0,9], dtype=int64), 0)

	# get matrix
	a_out = a.get_feature_matrix()

	assert(all(a_out==A))
	return a_out

if __name__=='__main__':
	print('dense_longint')
	features_dense_longint_modular(*parameter_list[0])



#!/usr/bin/python

import numpy as np
import matplotlib.pyplot as plt

from modshogun import RegressionLabels, RealFeatures
from modshogun import LeastAngleRegression, LinearRidgeRegression, LeastSquaresRegression
from modshogun import MeanSquaredError

# we compare LASSO with ordinary least-squares (OLE)
# in the ideal case, the MSE of OLE should coincide
# with LASSO at the end of the path
#
# if OLE is unstable, we may use RidgeRegression (with
# a small regularization coefficient) to simulate OLE
use_ridge = False

np.random.seed(1024)

n           = 200
ntrain      = 100
p           = 7
correlation = 0.6

mean = np.zeros(p)
cov  = correlation*np.ones((p,p)) + (1-correlation)*np.eye(p)

Xall = np.random.multivariate_normal(mean, cov, n)

# model is the linear combination of the first three variables plus noise
yall = 2*Xall[:,0] + 5*Xall[:,1] + -3*Xall[:,2] + 0.5*np.random.randn(n)

X = Xall[0:ntrain,:]
y = yall[0:ntrain]

Xtest = Xall[ntrain:,:]
ytest = yall[ntrain:]

# preprocess data
for i in xrange(p):
    X[:,i] -= np.mean(X[:,i])
    X[:,i] /= np.linalg.norm(X[:,i])
y -= np.mean(y)

# train LASSO
LeastAngleRegression = LeastAngleRegression()
LeastAngleRegression.set_labels(RegressionLabels(y))
LeastAngleRegression.train(RealFeatures(X.T))

# train ordinary LSR
if use_ridge:
    lsr = LinearRidgeRegression(0.01, RealFeatures(X.T), Labels(y))
    lsr.train()
else:
    lsr = LeastSquaresRegression()
    lsr.set_labels(RegressionLabels(y))
    lsr.train(RealFeatures(X.T))

# gather LASSO path
path = np.zeros((p, LeastAngleRegression.get_path_size()))
for i in xrange(path.shape[1]):
    path[:,i] = LeastAngleRegression.get_w(i)

evaluator = MeanSquaredError()

# apply on training data
mse_train = np.zeros(LeastAngleRegression.get_path_size())
for i in xrange(mse_train.shape[0]):
    LeastAngleRegression.switch_w(i)
    ypred = LeastAngleRegression.apply(RealFeatures(X.T))
    mse_train[i] = evaluator.evaluate(ypred, RegressionLabels(y))
ypred = lsr.apply(RealFeatures(X.T))
mse_train_lsr = evaluator.evaluate(ypred, RegressionLabels(y))

# apply on test data
mse_test = np.zeros(LeastAngleRegression.get_path_size())
for i in xrange(mse_test.shape[0]):
    LeastAngleRegression.switch_w(i)
    ypred = LeastAngleRegression.apply(RealFeatures(Xtest.T))
    mse_test[i] = evaluator.evaluate(ypred, RegressionLabels(y))
ypred = lsr.apply(RealFeatures(Xtest.T))
mse_test_lsr = evaluator.evaluate(ypred, RegressionLabels(y))

fig = plt.figure()
ax_path = fig.add_subplot(1,2,1)
plt.plot(xrange(path.shape[1]), path.T, '.-')
plt.legend(['%d' % (x+1) for x in xrange(path.shape[0])])
plt.xlabel('iteration')
plt.title('LASSO path')

ax_tr   = fig.add_subplot(2,2,2)
plt.plot(range(mse_train.shape[0])[1:], mse_train[1:], 'k.-')
plt.plot(range(mse_train.shape[0])[1:], np.zeros(mse_train.shape[0])[1:] + mse_train_lsr, 'r-')
plt.legend(('LASSO', 'LeastSquares'))
plt.xlabel('# of non-zero variables')
plt.ylabel('MSE')
plt.title('MSE on training data')

ax_tt   = fig.add_subplot(2,2,4)
plt.plot(range(mse_test.shape[0])[1:], mse_test[1:], 'k.-')
plt.plot(range(mse_test.shape[0])[1:], np.zeros(mse_test.shape[0])[1:] + mse_test_lsr, 'r-')
plt.legend(('LASSO', 'LeastSquares'), loc='lower right')
plt.xlabel('# of non-zero variables')
plt.ylabel('MSE')
plt.title('MSE on test data')

plt.show()




"""Common operations related to file handling"""

import os
from numpy import int, long, float, double, ushort, uint16, ubyte, short, matrix, int32, int64, uint32, uint64

import featop
import dataop
import category

DIR_OUTPUT='data'
EXT_OUTPUT='.m'


def _get_matrix (name, kmatrix):
	"""Converts a numpy matrix into a matrix digestable by e.g. matlab.

	@param name Name of the matrix
	@param kmatrix The matrix
	@return String which contains a matrix digestable by e.g. matlab
	"""

	line=list()
	list_numbers=(int, long, float, double, ubyte, ushort, short, uint16, int32, int64, uint32, uint64)
	matrix=[]
	is_string=True

	try:
		# assume, all elements have same data type
		if isinstance(kmatrix[0, 0], list_numbers):
			is_string=False

		for x in range(kmatrix.shape[0]):
			for y in range(kmatrix.shape[1]):
				if is_string:
					line.append("'%s'" % kmatrix[x, y])
				else:
					line.append('%.9g' % kmatrix[x, y])
			matrix.append(', '.join(line))
			line=list()
	except IndexError:
		if isinstance(kmatrix[0], list_numbers):
			is_string=False

		for x in range(kmatrix.shape[0]):
			if is_string:
				line.append("'%s'" % kmatrix[x])
			else:
				line.append('%.9g' % kmatrix[x])
		matrix.append(', '.join(line))
		line=list()

	matrix=';'.join(matrix)

	if is_string:
		matrix=''.join([name, ' = {', matrix, '}'])
	else:
		matrix=''.join([name, ' = [', matrix, ']'])

	return matrix.replace('\n', '')


def _is_excluded_from_filename (key):
	"""
	Determine if given key's value shall not be part of the filename.

	@param key name of the value to check for.
	@return true if to be excluded, false otherwise
	"""

	if (key.find('feature_')!=-1 or
		key.find('accuracy')!=-1 or
		key.find('data_')!=-1 or
		key.find('normalizer')!=-1 or
		key=='name' or
		key=='init_random' or
		key=='regression_type' or
		key=='regression_bias' or
		key=='regression_alpha_sum' or
		key=='regression_sv_sum' or
		key=='distribution_likelihood' or
		key=='distribution_derivatives' or
		key=='distribution_best_path' or
		key=='distribution_best_path_state' or
		key=='classifier_bias' or
		key=='classifier_label_type' or
		key=='classifier_alpha_sum' or
		key=='classifier_sv_sum' or
		key=='classifier_type'):
		return True
	else:
		return False


def _get_filename (catID, out):
	"""
	Return filename for testcase data's output.

	@param catID ID of the category
	@param out data to be written into file
	@return string with the filename
	"""

	params=[]

	name=category.get_name(catID)
	for key, val in out.iteritems():
		if _is_excluded_from_filename(key):
			continue
		if key==name:
			continue

		cname=val.__class__.__name__
		if cname=='bool' or cname=='float' or cname=='int' or cname=='str':
			val=str(val)
			val=val.replace('-', 'n')
			val=val.replace('+', 'p')
			params.append(val)

	params='_'.join(params).replace('.', '')
	if len(params)>0:
		params='_'+params
	else: # otherwise problems with one interface (FIXME: find out which)
		params='_fnord'

	dir=DIR_OUTPUT+os.sep+category.get_as_string(catID)+os.sep
	return dir+out[name]+params+EXT_OUTPUT


def _loop_args(args, prefix):
	"""
	Loop through all arguments in given dict and add to appropriately to
	output data.

	@param params various clustering parameters
	@param prefix prefix for parameter's name, e.g. 'preproc_'
	@return dict containing testcase data ready to be written to file
	"""

	out={}
	for i in xrange(len(args['key'])):
		try:
			name=prefix+'arg'+str(i)+'_'+args['key'][i]
		except IndexError:
			break

		cname=args['val'][i].__class__.__name__
		if cname=='int' or cname=='float' or cname=='bool' or cname=='str':
			out[name]=args['val'][i]
		else:
			out[name]=cname

	return out


def _get_output_classifier (params, prefix=''):
	"""
	Classifier-specific gathering of output data

	@param params various classifier parameters
	@param prefix prefix for parameter's name, e.g. 'classifier_'
	@return dict containing testcase data ready to be written to file
	"""

	out={}
	for key, val in params.iteritems():
		if key!='data' and val is not None:
			out[prefix+key]=val

	return out


def _get_output_clustering (params, prefix=''):
	"""
	Clustering-specific gathering of output data

	@param params various clustering parameters
	@param prefix prefix for parameter's name, e.g. 'clustering_'
	@return dict containing testcase data ready to be written to file
	"""

	out={}
	for key, val in params.iteritems():
		out[prefix+key]=val

	return out


def _get_output_distance (params, prefix=''):
	"""
	Distance-specific gathering of outdata

	@param params various distance parameters
	@param prefix prefix for parameter's name, e.g. 'subkernel'
	@return dict containing testcase data ready to be written to file
	"""

	if not params.has_key('args'):
		return {}

	return _loop_args(params['args'], prefix)


def _get_output_distribution (params, prefix=''):
	"""
	Clustering-specific gathering of output data

	@param params various clustering parameters
	@param prefix prefix for parameter's name, e.g. 'clustering_'
	@return dict containing testcase data ready to be written to file
	"""

	out={}
	for key, val in params.iteritems():
		if key!='data' and val is not None:
			out[prefix+key]=val

	return out


def _get_output_kernel (params, prefix=''):
	"""
	Kernel-specific gathering of outdata

	@param params various kernel parameters
	@param prefix prefix for parameter's name, e.g. 'subkernel'
	@return dict containing testcase data ready to be written to file
	"""

	out={}

	if params.has_key('normalizer'):
		out[prefix+'normalizer']=params['normalizer'].__class__.__name__

	if not params.has_key('args'):
		return out

	if params['name']=='AUC':
		# remove element 'subkernel'
		params['args']['key']=(params['args']['key'][0],)

	out.update(_loop_args(params['args'], prefix))
	return  out


def _get_output_preproc (params, prefix=''):
	"""
	Preproc-specific gathering of output data

	@param params various preproc parameters
	@param prefix prefix for parameter's name, e.g. 'preproc_'
	@return dict containing testcase data ready to be written to file
	"""

	if not params.has_key('args'):
		return {}

	return _loop_args(params['args'], prefix)


def _get_output_regression (params, prefix=''):
	"""
	Regression-specific gathering of output data

	@param params various regression parameters
	@param prefix prefix for parameter's name, e.g. 'regression_'
	@return dict containing testcase data ready to be written to file
	"""

	out={}
	for key, val in params.iteritems():
		out[prefix+key]=val

	return out



############################################################################
# public
############################################################################

def write (catID, out):
	"""
	Write given testcase data to a file.

	@param cat ID of the category, like category.KERNEL
	@param out data to be written into file
	@return Success of operation
	"""

	fnam=_get_filename(catID, out)
	print 'Writing for '+category.get_as_string(catID).upper()+': '+ \
		os.path.basename(fnam)

	dirname=os.path.dirname(fnam)
	if not os.path.isdir(dirname):
		os.mkdir(dirname)

	mfile=open(fnam, mode='w')
	for key, val in out.iteritems():
		cname=val.__class__.__name__
		if cname=='bool' or cname=='str':
			mfile.write("%s = '%s';\n"%(key, val))
		elif cname=='ndarray' or cname=='matrix':
			mfile.write("%s;\n"%_get_matrix(key, val))
		else:
			mfile.write("%s = %s;\n"%(key, val))
	mfile.close()

	return True


def clean_dir_outdata ():
	"""
	Remove all old testdata files.

	@return Success of operation
	"""

	success=True

	for dname in os.listdir(DIR_OUTPUT):
		if dname=='.svn' or not os.path.isdir(DIR_OUTPUT+os.sep+dname):
			continue

		for fname in os.listdir(DIR_OUTPUT+os.sep+dname):
			if not fname.endswith(EXT_OUTPUT):
				continue

			target=DIR_OUTPUT+os.sep+dname+os.sep+fname
			if os.path.exists(target):
				os.remove(target)
				# os.remove returns False on removal???
				#print 'Could not remove file "%s"'%target
				#success=False

	return success


def get_output (catID, params, prefix=''):
	"""
	Return output data to be written into the testcase's file.

	After computations, the gathered data is structured and
	put into one data structure which can conveniently be written to a
	file that will represent the testcase.

	@param catID ID of entity's category, e.g. category.DISTANCE
	@param params hash with parameters to entity
	@param prefix prefix for parameter's name, e.g. 'subkernel'
	@return Dict containing testcase data ready to be written to file
	"""

	out={}
	prefix=category.get_as_string(catID)+'_'+prefix
	if catID==category.CLASSIFIER:
		out=_get_output_classifier(params, prefix)
	elif catID==category.CLUSTERING:
		out=_get_output_distribution(params, prefix)
	elif catID==category.DISTANCE:
		out=_get_output_distance(params, prefix)
	elif catID==category.DISTRIBUTION:
		out=_get_output_distribution(params, prefix)
	elif catID==category.KERNEL:
		out=_get_output_kernel(params, prefix)
	elif catID==category.PREPROC:
		out=_get_output_preproc(params, prefix)
	elif catID==category.REGRESSION:
		out=_get_output_regression(params, prefix)
	else:
		return out

	out[prefix+'name']=params['name']
	if params.has_key('accuracy'):
		out[prefix+'accuracy']=params['accuracy']

	# example data
	if params.has_key('data'):
		out[prefix+'data_train']=matrix(params['data']['train'])
		out[prefix+'data_test']=matrix(params['data']['test'])


	# params wrt feature class & type
	if params.has_key('feature_class'):
		fclass=params['feature_class']
		ftype=params['feature_type']
		out[prefix+'feature_class']=fclass
		out[prefix+'feature_type']=ftype

		if fclass=='string' or (fclass=='simple' and ftype=='Char'):
			if params.has_key('alphabet'):
				out[prefix+'alphabet']=params['alphabet']
			else:
				out[prefix+'alphabet']='DNA'
			out[prefix+'seqlen']=dataop.LEN_SEQ

		elif fclass=='simple' and ftype=='Byte':
			out[prefix+'alphabet']='RAWBYTE'
			out[prefix+'seqlen']=dataop.LEN_SEQ

		elif fclass=='string_complex':
			if params.has_key('alphabet'):
				out[prefix+'alphabet']=params['alphabet']
			else:
				out[prefix+'alphabet']='DNA'
			if params.has_key('order'):
				out[prefix+'order']=params['order']
			else:
				out[prefix+'order']=featop.WORDSTRING_ORDER
			if params.has_key('gap'):
				out[prefix+'gap']=params['gap']
			else:
				out[prefix+'gap']=featop.WORDSTRING_GAP
			if params.has_key('reverse'):
				out[prefix+'reverse']=params['reverse']
			else:
				out[prefix+'reverse']=featop.WORDSTRING_REVERSE
			if params.has_key('seqlen'):
				out[prefix+'seqlen']=params['seqlen']
			else:
				out[prefix+'seqlen']=dataop.LEN_SEQ

	out['init_random']=dataop.INIT_RANDOM

	return out



#!/usr/bin/env python

#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  Written (W) 2013 Evangelos Anagnostopoulos
#

def parse_arguments():
	import argparse
	parser = argparse.ArgumentParser(description=
		"Solve binary classification problems stored in libsvm format, "
		"using Random Fourier features and SVMOcas")
	parser.add_argument('--dataset', required=True, type=str,
					help='Path to training dataset in LibSVM format.')
	parser.add_argument('--testset', type=str,
					help='Path to test dataset in LibSVM format.')
	parser.add_argument('-D', default=300, type=int,
					help='The number of samples to use')
	parser.add_argument('-C', default=0.1, type=float,
					help='SVMOcas regularization constant')
	parser.add_argument('--epsilon', default=0.01, type=float,
					help='SVMOcas epsilon parameter')
	parser.add_argument('--width', default=8, type=float,
					help='Width of the Gaussian Kernel to approximate')
	parser.add_argument('--dimension', type=int,
					help='Dimension of input dataset')

	return parser.parse_args()

def evaluate(predicted_labels, labels, prefix="Results"):
	from modshogun import PRCEvaluation, ROCEvaluation, AccuracyMeasure

	prc_evaluator = PRCEvaluation()
	roc_evaluator = ROCEvaluation()
	acc_evaluator = AccuracyMeasure()

	auPRC = prc_evaluator.evaluate(predicted_labels, labels)
	auROC = roc_evaluator.evaluate(predicted_labels, labels)
	acc = acc_evaluator.evaluate(predicted_labels, labels)

	print ('{0}: auPRC = {1:.5f}, auROC = {2:.5f}, acc = {3:.5f} '+
				'({4}% incorrectly classified)').format(
				prefix, auPRC, auROC, acc, (1-acc)*100)

def load_sparse_data(filename, dimension=None):
	input_file = LibSVMFile(args.dataset)
	sparse_feats = SparseRealFeatures()
	label_array = sparse_feats.load_with_labels(input_file)
	labels = BinaryLabels(label_array)

	if dimension!=None:
		sparse_feats.set_num_features(dimension)

	return {'data':sparse_feats, 'labels':labels}

if __name__=='__main__':
	from modshogun import SparseRealFeatures, RandomFourierDotFeatures, GAUSSIAN
	from modshogun import LibSVMFile, BinaryLabels, SVMOcas
	from modshogun import Time
	from numpy import array

	args = parse_arguments()

	print 'Loading training data...'
	sparse_data = load_sparse_data(args.dataset,args.dimension)

	kernel_params = array([args.width], dtype=float)
	rf_feats = RandomFourierDotFeatures(sparse_data['data'], args.D, GAUSSIAN,
				kernel_params)

	svm = SVMOcas(args.C, rf_feats, sparse_data['labels'])
	svm.set_epsilon(args.epsilon)
	print 'Starting training.'
	timer = Time()
	svm.train()
	timer.stop()
	print 'Training completed, took {0:.2f}s.'.format(timer.time_diff_sec())

	predicted_labels = svm.apply()
	evaluate(predicted_labels, sparse_data['labels'], 'Training results')

	if args.testset!=None:
		random_coef = rf_feats.get_random_coefficients()
		# removing current dataset from memory in order to load the test dataset,
		# to avoid running out of memory
		rf_feats = None
		svm.set_features(None)
		svm.set_labels(None)
		sparse_data = None

		print 'Loading test data...'
		sparse_data = load_sparse_data(args.testset, args.dimension)
		rf_feats = RandomFourierDotFeatures(sparse_data['data'], args.D, GAUSSIAN,
					kernel_params, random_coef)
		predicted_labels = svm.apply(rf_feats)
		evaluate(predicted_labels, sparse_data['labels'], 'Test results')



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
parameter_list=[[traindna,testdna,3,0,'n'],[traindna,testdna,4,0,'n']]

def distance_canberraword (fm_train_dna=traindna,fm_test_dna=testdna,order=3,
			    gap=0,reverse='n'):

	sg('set_distance', 'CANBERRA', 'WORD')
	sg('add_preproc', 'SORTWORDSTRING')
	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TRAIN')
	dm=sg('get_distance_matrix', 'TRAIN')
	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TEST')
	dm=sg('get_distance_matrix', 'TEST')
	return dm

if __name__=='__main__':
	print('CanberraWordDistance')
	distance_canberraword(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
parameter_list=[[traindat,testdat],[traindat,testdat]]

def distance_chebyshew (fm_train_real=traindat,fm_test_real=testdat):
	sg('set_distance', 'CHEBYSHEW', 'REAL')
	sg('set_features', 'TRAIN', fm_train_real)
	dm=sg('get_distance_matrix', 'TRAIN')
	sg('set_features', 'TEST', fm_test_real)
	dm=sg('get_distance_matrix', 'TEST')
	return dm

if __name__=='__main__':
	print('ChebyshewMetric')
	distance_chebyshew(*parameter_list[0])



#!/usr/bin/env python
from modshogun import StreamingVwFile
from modshogun import StreamingVwCacheFile
from modshogun import T_SVMLIGHT
from modshogun import StreamingVwFeatures
from modshogun import VowpalWabbit

parameter_list=[['../data/fm_train_sparsereal.dat']]

def streaming_vw_createcache_modular (fname):
	# First creates a binary cache from an ascii data file.
	# and then trains using the StreamingVwCacheFile as input

	# Open the input file as a StreamingVwFile
	input_file = StreamingVwFile(fname)
	# Default file name will be vw_cache.dat.cache
	input_file.set_write_to_cache(True)

	# Tell VW that the file is in SVMLight format
	# Supported types are T_DENSE, T_SVMLIGHT and T_VW
	input_file.set_parser_type(T_SVMLIGHT)

	## Create a StreamingVwFeatures object, `True' indicating the examples are labelled
	#features = StreamingVwFeatures(input_file, True, 1024)

	## Create a VW object from the features
	#vw = VowpalWabbit(features)
	#vw.set_no_training(True)

	## Train (in this case does nothing but run over all examples)
	#vw.train()

	##Finally Train using the generated cache file

	## Open the input cache file as a StreamingVwCacheFile
	#input_file = StreamingVwCacheFile("vw_cache.dat.cache");

	## The rest is exactly as for normal input
	#features = StreamingVwFeatures(input_file, True, 1024);
	#vw = VowpalWabbit(features)
	#vw.train()
	##return vw

if __name__ == "__main__":
	streaming_vw_createcache_modular(*parameter_list[0])



#!/usr/bin/env python
import numpy

from modshogun import StringCharFeatures, BinaryLabels, DNA
from modshogun import WeightedDegreeStringKernel
from modshogun import SVMLight, DomainAdaptationSVM, MSG_DEBUG

traindna = ['CGCACGTACGTAGCTCGAT',
		      'CGACGTAGTCGTAGTCGTA',
		      'CGACGGGGGGGGGGTCGTA',
		      'CGACCTAGTCGTAGTCGTA',
		      'CGACCACAGTTATATAGTA',
		      'CGACGTAGTCGTAGTCGTA',
		      'CGACGTAGTTTTTTTCGTA',
		      'CGACGTAGTCGTAGCCCCA',
		      'CAAAAAAAAAAAAAAAATA',
		      'CGACGGGGGGGGGGGCGTA']
label_traindna = numpy.array(5*[-1.0] + 5*[1.0])
testdna = ['AGCACGTACGTAGCTCGAT',
		      'AGACGTAGTCGTAGTCGTA',
		      'CAACGGGGGGGGGGTCGTA',
		      'CGACCTAGTCGTAGTCGTA',
		      'CGAACACAGTTATATAGTA',
		      'CGACCTAGTCGTAGTCGTA',
		      'CGACGTGGGGTTTTTCGTA',
		      'CGACGTAGTCCCAGCCCCA',
		      'CAAAAAAAAAAAACCAATA',
		      'CGACGGCCGGGGGGGCGTA']
label_testdna = numpy.array(5*[-1.0] + 5*[1.0])


traindna2 = ['AGACAGTCAGTCGATAGCT',
		      'AGCAGTCGTAGTCGTAGTC',
		      'AGCAGGGGGGGGGGTAGTC',
		      'AGCAATCGTAGTCGTAGTC',
		      'AGCAACACGTTCTCTCGTC',
		      'AGCAGTCGTAGTCGTAGTC',
		      'AGCAGTCGTTTTTTTAGTC',
		      'AGCAGTCGTAGTCGAAAAC',
		      'ACCCCCCCCCCCCCCCCTC',
		      'AGCAGGGGGGGGGGGAGTC']
label_traindna2 = numpy.array(5*[-1.0] + 5*[1.0])
testdna2 = ['CGACAGTCAGTCGATAGCT',
		      'CGCAGTCGTAGTCGTAGTC',
		      'ACCAGGGGGGGGGGTAGTC',
		      'AGCAATCGTAGTCGTAGTC',
		      'AGCCACACGTTCTCTCGTC',
		      'AGCAATCGTAGTCGTAGTC',
		      'AGCAGTGGGGTTTTTAGTC',
		      'AGCAGTCGTAAACGAAAAC',
		      'ACCCCCCCCCCCCAACCTC',
		      'AGCAGGAAGGGGGGGAGTC']
label_testdna2 = numpy.array(5*[-1.0] + 5*[1.0])

parameter_list = [[traindna,testdna,label_traindna,label_testdna,traindna2,label_traindna2, \
                       testdna2,label_testdna2,1,3],[traindna,testdna,label_traindna,label_testdna,traindna2,label_traindna2, \
                       testdna2,label_testdna2,2,5]]

def classifier_domainadaptationsvm_modular (fm_train_dna=traindna,fm_test_dna=testdna, \
                                                label_train_dna=label_traindna, \
                                               label_test_dna=label_testdna,fm_train_dna2=traindna2,fm_test_dna2=testdna2, \
                                               label_train_dna2=label_traindna2,label_test_dna2=label_testdna2,C=1,degree=3):




	feats_train = StringCharFeatures(fm_train_dna, DNA)
	feats_test = StringCharFeatures(fm_test_dna, DNA)
	kernel = WeightedDegreeStringKernel(feats_train, feats_train, degree)
	labels = BinaryLabels(label_train_dna)
	svm = SVMLight(C, kernel, labels)
	svm.train()
	#svm.io.set_loglevel(MSG_DEBUG)

	#####################################

	#print("obtaining DA SVM from previously trained SVM")

	feats_train2 = StringCharFeatures(fm_train_dna, DNA)
	feats_test2 = StringCharFeatures(fm_test_dna, DNA)
	kernel2 = WeightedDegreeStringKernel(feats_train, feats_train, degree)
	labels2 = BinaryLabels(label_train_dna)

	# we regularize against the previously obtained solution
	dasvm = DomainAdaptationSVM(C, kernel2, labels2, svm, 1.0)
	dasvm.train()

	out = dasvm.apply_binary(feats_test2)

	return out #,dasvm TODO

if __name__=='__main__':
	print('SVMLight')
	classifier_domainadaptationsvm_modular(*parameter_list[0])



#!/usr/bin/env python

from numpy import double, fromfile, loadtxt

class LoadMatrix:
	def __init__(self):
		pass


	def load_numbers(self, filename):
		return loadtxt(filename).T

	def load_dna(self, filename):
		fh=open(filename, 'r');
		matrix=[]

		for line in fh:
			matrix.append(line[:-1])

		return matrix


	def load_cubes(self, filename):
		fh=open(filename, 'r');
		matrix=[]

		for line in fh:
			matrix.append(line.split(' ')[0][:-1])

		fh.close()

		return matrix


	def load_labels(self, filename):
		return fromfile(filename, dtype=double, sep=' ')



# File   : $HeadURL$
# Version: $Id$

import gtk
import numpy as np

import common as com
from QuadrWidget import QuadrWidget

class FigureWidget(QuadrWidget):
    THICKNESS_FRAC = 0.03

    def __init__(self, go_func, go_args):
        QuadrWidget.__init__(self)
        self.add_events(gtk.gdk.ALL_EVENTS_MASK)

        self.drag = False
        self.coords = []

        self.go_func = go_func
        self.go_args = go_args

        self.connect("expose_event", FigureWidget.on_redraw)
        self.connect("button-press-event", FigureWidget.on_press)
        self.connect("button-release-event", FigureWidget.on_release)
        self.connect("motion-notify-event", FigureWidget.on_motion)
        self.connect("realize", FigureWidget.on_realize)

    def on_realize(self):
        self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.CROSSHAIR))

    def on_press(self, event):
        if event.button == com.BUTTON_RIGHT:
            self.clear_coords()
            self.drag = False

        if event.button == com.BUTTON_MIDDLE:
            self.go_func(*self.go_args)
            self.drag = False

        if event.button != com.BUTTON_LEFT:
            return False

        self.drag = True
        self.coords.insert(0, [])

        # Two times to add a zero-length line
        self.on_motion(event)
        self.on_motion(event)

        return False

    def on_release(self, event):
        if event.button != com.BUTTON_LEFT:
            return False

        self.drag = False
        return False

    def on_motion(self, event):
        if not self.drag:
            return False

        width = self.window.get_size()[0]
        height = self.window.get_size()[1]
        x = event.x/width if event.x < width else com.NEAR_ONE_NEG
        y = event.y/height if event.y < height else com.NEAR_ONE_NEG
        x = 0 if x < 0 else x
        y = 0 if y < 0 else y

        self.coords[0].append((x, y))

        self.update()

        return False

    def on_redraw(self, event):
        gc = self.style.fg_gc[self.state]
        w = self.window
        width = w.get_size()[0]
        height = w.get_size()[1]

        # Backup graphic context
        self.default_fg = gc.foreground
        self.line_width = gc.line_width
        self.line_style = gc.line_style
        self.cap_style = gc.cap_style
        self.join_style = gc.join_style

        # Background
        gc.set_rgb_fg_color(com.COLOR_WHITE)
        w.draw_rectangle(gc, True, 0, 0, width-1, height-1)
        gc.set_rgb_fg_color(com.COLOR_BLACK)
        w.draw_rectangle(gc, False, 0, 0, width-1, height-1)

        # Data
        gc.set_line_attributes(int(height*self.THICKNESS_FRAC),
                               gtk.gdk.LINE_SOLID, gtk.gdk.CAP_ROUND,
                               gtk.gdk.JOIN_ROUND)
        for poly in self.coords:
            w.draw_lines(gc, map(
                    lambda coord:
                        (int(coord[0]*width), int(coord[1]*height)),
                    poly))

        # Recovering graphic context
        gc.line_width = self.line_width
        gc.line_style = self.line_style
        gc.cap_style = self.cap_style
        gc.join_style = self.join_style
        gc.foreground = self.default_fg

        return False

    def get_coords(self):
        result = map(lambda line: np.array(line), self.coords)

        result = map(lambda line: np.transpose(line), result)

        minx = 2.0
        miny = 2.0
        for line in result:
            minx = min(minx, min(line[0]))
            miny = min(miny, min(line[1]))
        for line in result:
            line[0] -= minx
            line[1] -= miny

        maxxy = 0.0
        for line in result: maxxy = max(maxxy, line.max())
        for line in result: line /= maxxy + com.NEAR_ZERO_POS

        maxx = 0.0
        maxy = 0.0
        for line in result:
            maxx = max(maxx, max(line[0]))
            maxy = max(maxy, max(line[1]))
        for line in result:
            line[0] += (1 - maxx)/2
            line[1] += (1 - maxy)/2

        result = map(lambda line: np.transpose(line), result)
        return result

    def clear_coords(self):
        self.coords = []
        self.update()



"""Generator for Distance"""

import shogun.Distance as distance

import fileop
import dataop
import featop
import category


def _compute (feats, params):
	"""Compute a distance and gather result data.

	@param feats Train and test features
	@param params dict with parameters to distance
	"""

	fun=eval('distance.'+params['name'])
	if params.has_key('args'):
		dist=fun(feats['train'], feats['train'], *params['args']['val'])
	else:
		dist=fun(feats['train'], feats['train'])
	dm_train=dist.get_distance_matrix()
	dist.init(feats['train'], feats['test'])
	dm_test=dist.get_distance_matrix()

	output={
		'distance_matrix_train':dm_train,
		'distance_matrix_test':dm_test,
	}
	output.update(fileop.get_output(category.DISTANCE, params))

	fileop.write(category.DISTANCE, output)

def _run_feats_real ():
	"""Run distances with RealFeatures."""

	params={
		'accuracy': 1e-8,
		'feature_class': 'simple',
		'feature_type': 'Real',
		'data': dataop.get_rand()
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	params['name']='EuclidianDistance'
	_compute(feats, params)
	params['name']='CanberraMetric'
	_compute(feats, params)
	params['name']='ChebyshewMetric'
	_compute(feats, params)
	params['name']='GeodesicMetric'
	_compute(feats, params)
	params['name']='JensenMetric'
	_compute(feats, params)
	params['name']='ManhattanMetric'
	_compute(feats, params)
	params['name']='BrayCurtisDistance'
	_compute(feats, params)
	params['name']='ChiSquareDistance'
	_compute(feats, params)
	params['name']='CosineDistance'
	_compute(feats, params)
	params['name']='TanimotoDistance'
	_compute(feats, params)
	params['name']='ManhattanMetric'
	_compute(feats, params)
	params['name']='MinkowskiMetric'
	params['args']={'key': ('k',), 'val': (1.3,)}
	_compute(feats, params)

	params['name']='SparseEuclidianDistance'
	params['accuracy']=1e-7
	del params['args']
	feats=featop.get_features(
		params['feature_class'], params['feature_type'],
		params['data'], sparse=True)
	_compute(feats, params)


def _run_feats_string_complex ():
	"""Run distances with complex StringFeatures, like WordString."""

	params={
		'accuracy': 1e-7,
		'feature_class': 'string_complex',
		'feature_type': 'Word',
		'data': dataop.get_dna(num_vec_test=dataop.NUM_VEC_TRAIN+42)
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])

	params['name']='CanberraWordDistance'
	_compute(feats, params)

	params['accuracy']=1e-8
	params['name']='ManhattanWordDistance'
	_compute(feats, params)

	params['name']='HammingWordDistance'
	params['args']={'key': ('use_sign',), 'val': (False,)}
	_compute(feats, params)
	params['name']='HammingWordDistance'
	params['args']={'key': ('use_sign',), 'val': (True,)}
	_compute(feats, params)


def run ():
	"""Run generator for all distances."""

	_run_feats_real()
	_run_feats_string_complex()




import org.shogun.*;
import org.jblas.*;

public class converter_locallylinearembedding_modular {
	static {
		System.loadLibrary("modshogun");
	}

	public static void main(String argv[]) {
		modshogun.init_shogun_with_defaults();

		DoubleMatrix data = Load.load_numbers("../data/fm_train_real.dat");

		RealFeatures features = new RealFeatures(data);
		LocallyLinearEmbedding lle = new LocallyLinearEmbedding();
		lle.set_target_dim(1);
		RealFeatures embedding = lle.embed(features);

		modshogun.exit_shogun();
	}
}



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindat,testdat,5,5,1],[traindat,testdat,5,3,2]]

def kernel_simple_locality_improved_string_modular (fm_train_dna=traindat,fm_test_dna=testdat,
	length=5,inner_degree=5,outer_degree=1 ):

	from modshogun import StringCharFeatures, DNA
	from modshogun import SimpleLocalityImprovedStringKernel, MSG_DEBUG

	feats_train=StringCharFeatures(fm_train_dna, DNA)
	#feats_train.io.set_loglevel(MSG_DEBUG)
	feats_test=StringCharFeatures(fm_test_dna, DNA)


	kernel=SimpleLocalityImprovedStringKernel(
		feats_train, feats_train, length, inner_degree, outer_degree)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('SimpleLocalityImprovedString')
	kernel_simple_locality_improved_string_modular(*parameter_list[0])



#!/usr/bin/env python

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import bz2
import time
import sys
from splicesites.utils import create_dataset, create_modsel
from esvm.utils import calcroc
from esvm.experiment import crossvalidation
from esvm.mldata import init_datasetfile
from numpy.linalg import norm
import numpy

def write_results(f, results):
    """
    Write out results
    """

    f.write('Kernel\tParameters\tC\tauROC\n');
    for i in xrange(len(results)):
        C=results[i][0]
        k_param=results[i][1]
        param_name=k_param[1]['name']
        kernel=k_param[0]
        if kernel.endswith('2'):
            kernel=kernel[:-1]
        kernel_parameters= param_name + '=' + `k_param[1][param_name]`
        perf = 100*results[i][2]

        f.write('%s\t' % kernel)
        f.write('%s\t' % kernel_parameters)
        f.write('C=%2.2f\t' % C)
        f.write('%2.1f%%\n' % perf)

def normalize(examples, subtract_mean=False, divide_std=False, rescale=False, norm_one=False):
    """
    Scale GC data to ... (be on a ball? just const? 0 mean, std 1?)
    """

    if subtract_mean:
        # mean = 0.0
        mean=numpy.mean(examples, axis=1)
        for i in xrange(examples.shape[1]):
            examples[:,i]-=mean

    if divide_std:
        # std = 1.0
        std=numpy.std(examples, axis=1)
        for i in xrange(examples.shape[1]):
            examples[:,i]/=(std+1e-10)

    if rescale:
        # scale to have on average 1 on linear kernel diagonal
        scale=numpy.sqrt(numpy.mean(numpy.diag(numpy.mat(examples).T*numpy.mat(examples))))
        examples/=scale

    if norm_one:
        # ball/circle
        for i in xrange(examples.shape[1]):
            examples[:,i]/=norm(examples[:,i])

    return examples ;

def run_single_experiment(results, num_fold_cv, kernelname, kparam, C, examples, labels):
    """
    Run a single experiment, i.e. for a fixed kernel and parameters
    do num_fold cross-validation
    """

    param_name=kparam['name']
    print 'Running C =', C, kernelname.title(), 'Kernel with', param_name, '=', kparam[param_name]
    (all_outputs, all_split) = crossvalidation(num_fold_cv, kernelname, kparam, C, examples, labels, 'dna', 'A')
    results.append( (C, (kernelname, kparam), calcroc(all_outputs,labels)) )

def splice_example(Cs, gcfilename,seqfilename,seq2filename, plot=False):
    """
    For the data files, apply the set of kernels
    """
    # hyperparameters
    num_fold_cv = 5

    # The area under the receiver operating characteristic
    results=[]

    # Read datasets

    # GC features
    fp = init_datasetfile(gcfilename,'vec')
    (gc_examples,gc_labels) = fp.readlines()
    gc_examples = normalize(gc_examples, subtract_mean=True)

    if plot:
        from pylab import scatter,show
        color=['b','r']
        scatter(gc_examples[0,], gc_examples[1,], s=400*(gc_labels+2), c=''.join([ color[(int(i)+1)/2] for i in gc_labels]), alpha=0.1)
        show()

    # 2 sequence features
    fp = init_datasetfile(seq2filename,'mseq')
    (dna2_examples,dna2_labels) = fp.readlines()

    # DNA sequences
    fp = init_datasetfile(seqfilename,'seq')
    (dna_examples,dna_labels) = fp.readlines()


    #Define experiments to carry out

    experiments=(
    # Linear kernel on GC content
    ('linear', {'scale':1.0, 'name':'scale'}, (gc_examples, gc_labels)),

    # Polynomial kernel on GC content
    ( 'poly', {'degree':3, 'name':'degree', 'inhomogene':True, 'normal':True}, (gc_examples, gc_labels)),
    ( 'poly', {'degree':5, 'name':'degree', 'inhomogene':True, 'normal':True}, (gc_examples, gc_labels)),

    # Gaussian kernel on GC content
    ('gauss', {'width':100.0, 'name':'width'}, (gc_examples, gc_labels)),
    ('gauss', {'width':1.0, 'name':'width'}, (gc_examples, gc_labels)),
    ('gauss', {'width':0.01, 'name':'width'}, (gc_examples, gc_labels)),

    # Spectrum kernel on 2 dna sequences
    ('spec2', {'degree':1, 'name':'degree'}, (dna2_examples, dna2_labels)),
    ('spec2', {'degree':3, 'name':'degree'}, (dna2_examples, dna2_labels)),
    ('spec2', {'degree':5, 'name':'degree'}, (dna2_examples, dna2_labels)),

    # Cumulative Spectrum kernel on 2 dna sequences
    ('cumspec2', {'degree':1, 'name':'degree'}, (dna2_examples, dna2_labels)),
    ('cumspec2', {'degree':3, 'name':'degree'}, (dna2_examples, dna2_labels)),
    ('cumspec2', {'degree':5, 'name':'degree'}, (dna2_examples, dna2_labels)),

    # Weighted degree kernel on dna sequences
    ('wd', {'degree':1,'shift':0, 'name':'degree'}, (dna_examples, dna_labels)),
    ('wd', {'degree':3,'shift':0, 'name':'degree'}, (dna_examples, dna_labels)),
    ('wd', {'degree':5,'shift':0, 'name':'degree'}, (dna_examples, dna_labels))
    )


    if Cs is None:
        for C in (0.01, 0.1, 1, 2, 5, 10):
            for e in experiments:
                run_single_experiment(results, num_fold_cv, e[0], e[1], C, e[2][0], e[2][1])
    else:
        for i in xrange(len(experiments)):
            e=experiments[i]
            run_single_experiment(results, num_fold_cv, e[0], e[1], Cs[i], e[2][0], e[2][1])

    return results

def get_best_results(results):
    methods=('linear', 'poly', 'gauss', 'spec2', 'cumspec2', 'wd')
    best_result=[]
    for m in methods:
        params=set()
        for r in results:
            if r[1][0]==m:
                params.add(tuple(zip(r[1][1].keys(),r[1][1].values())))

        for p in params:
            m_result=0.0
            m_best=None
            for r in results:
                if r[1][0]==m and r[1][1]==dict(p) and r[2]>m_result:
                    m_result=r[2]
                    m_best=r
            best_result.append(m_best)
    return best_result


if __name__ == '__main__':
    if len(sys.argv) > 1:
        if sys.argv[1] == 'create_data':
            create_dataset()
            sys.exit(0)
        elif sys.argv[1] == 'mselect':
            results = splice_example(None, 'C_elegans_acc_modsel_gc.csv','C_elegans_acc_modsel_seq.csv','C_elegans_acc_modsel_seq2.csv')
            #results = splice_example(None, 'C_elegans_acc_gc.csv','C_elegans_acc_seq.csv','C_elegans_acc_seq2.csv')
            import pickle
            pickle.dump(results, file('mselect_result.pickle','w'))
            sys.exit(0)
        elif sys.argv[1] == 'get_best':
            import pickle
            results=pickle.load(file('mselect_result.pickle'))
            best_result=get_best_results(results)
            write_results(sys.stdout, best_result)

            #print 'Cs=[',
            #for e in best_result:
            #    print e[0], ",",
            #print ']'
            sys.exit(0)
        else:
            print "unknown argument"
            sys.exit(1)

    # without any argument
    starttime = time.time()

    Cs = [ 5, 10, 10, \
           5, 0.01, 10, \
           10, 10, 0.01, \
           10, 10, 10, \
           1, 1, 2 ]

    # run the experiment
    results = splice_example(Cs, 'C_elegans_acc_gc.csv','C_elegans_acc_seq.csv','C_elegans_acc_seq2.csv', False)

    stoptime = time.time()
    elapsedtime = time.strftime('Elapsed time (HH.MM:SS): %H.%M:%S',time.gmtime(stoptime-starttime))
    print elapsedtime

    write_results(file('results.txt','w'), results)
    for curline in file('results.txt').readlines():
        print curline.strip()

if __name__ == '__main__':
    if len(sys.argv) > 1:
        if sys.argv[1] == 'create_data':
            create_dataset()
        elif sys.argv[1] == 'create_modsel':
            create_modsel()
            sys.exit()
    #main()
    print 'results in results.txt'



#!/usr/bin/env python
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2012 Heiko Strathmann
# Copyright (C) 2012 Berlin Institute of Technology and Max-Planck-Society
#

from numpy.random import randn
from numpy import *

# generate some overlapping training vectors
num_vectors=5
vec_distance=1
traindat=concatenate((randn(2,num_vectors)-vec_distance,
	randn(2,num_vectors)+vec_distance), axis=1)
label_traindat=concatenate((-ones(num_vectors), ones(num_vectors)));

parameter_list = [[traindat,label_traindat]]

def evaluation_cross_validation_mkl_weight_storage(traindat=traindat, label_traindat=label_traindat):
    from modshogun import CrossValidation, CrossValidationResult
    from modshogun import CrossValidationPrintOutput
    from modshogun import CrossValidationMKLStorage
    from modshogun import ContingencyTableEvaluation, ACCURACY
    from modshogun import StratifiedCrossValidationSplitting
    from modshogun import BinaryLabels
    from modshogun import RealFeatures, CombinedFeatures
    from modshogun import GaussianKernel, CombinedKernel
    from modshogun import LibSVM, MKLClassification
    from modshogun import Statistics

    # training data, combined features all on same data
    features=RealFeatures(traindat)
    comb_features=CombinedFeatures()
    comb_features.append_feature_obj(features)
    comb_features.append_feature_obj(features)
    comb_features.append_feature_obj(features)
    labels=BinaryLabels(label_traindat)

    # kernel, different Gaussians combined
    kernel=CombinedKernel()
    kernel.append_kernel(GaussianKernel(10, 0.1))
    kernel.append_kernel(GaussianKernel(10, 1))
    kernel.append_kernel(GaussianKernel(10, 2))

    # create mkl using libsvm, due to a mem-bug, interleaved is not possible
    svm=MKLClassification(LibSVM());
    svm.set_interleaved_optimization_enabled(False);
    svm.set_kernel(kernel);

    # splitting strategy for 5 fold cross-validation (for classification its better
    # to use "StratifiedCrossValidation", but the standard
    # "StratifiedCrossValidationSplitting" is also available
    splitting_strategy=StratifiedCrossValidationSplitting(labels, 5)

    # evaluation method
    evaluation_criterium=ContingencyTableEvaluation(ACCURACY)

    # cross-validation instance
    cross_validation=CrossValidation(svm, comb_features, labels,
        splitting_strategy, evaluation_criterium)
    cross_validation.set_autolock(False)

    # append cross vlaidation output classes
    #cross_validation.add_cross_validation_output(CrossValidationPrintOutput())
    mkl_storage=CrossValidationMKLStorage()
    cross_validation.add_cross_validation_output(mkl_storage)
    cross_validation.set_num_runs(3)

    # perform cross-validation
    result=cross_validation.evaluate()

    # print mkl weights
    weights=mkl_storage.get_mkl_weights()
    #print "mkl weights during cross--validation"
    #print weights

    #print "mean per kernel"
    #print Statistics.matrix_mean(weights, False)

    #print "variance per kernel"
    #print Statistics.matrix_variance(weights, False)

    #print "std-dev per kernel"
    #print Statistics.matrix_std_deviation(weights, False)


if __name__=='__main__':
	print('Evaluation CrossValidationClassification')
	evaluation_cross_validation_mkl_weight_storage(*parameter_list[0])



#!/usr/bin/env python
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Copyright (C) 2012 Sergey Lisitsyn

from numpy import *
from numpy.random import randn

# generate some overlapping training vectors
num_vectors=100
vec_distance=1
traindat=concatenate((randn(2,num_vectors)-vec_distance,
	randn(2,num_vectors)+vec_distance), axis=1)
label_traindat=concatenate((-ones(num_vectors), ones(num_vectors)));

parameter_list = [[traindat,label_traindat]]

def modelselection_random_search_liblinear_modular (traindat=traindat, label_traindat=label_traindat):
    from modshogun import CrossValidation, CrossValidationResult
    from modshogun import ContingencyTableEvaluation, ACCURACY
    from modshogun import StratifiedCrossValidationSplitting
    from modshogun import RandomSearchModelSelection
    from modshogun import ModelSelectionParameters, R_EXP
    from modshogun import ParameterCombination
    from modshogun import BinaryLabels
    from modshogun import RealFeatures
    from modshogun import LibLinear, L2R_L2LOSS_SVC

    # build parameter tree to select C1 and C2
    param_tree_root=ModelSelectionParameters()
    c1=ModelSelectionParameters("C1");
    param_tree_root.append_child(c1)
    c1.build_values(-2.0, 2.0, R_EXP);

    c2=ModelSelectionParameters("C2");
    param_tree_root.append_child(c2);
    c2.build_values(-2.0, 2.0, R_EXP);

    # training data
    features=RealFeatures(traindat)
    labels=BinaryLabels(label_traindat)

    # classifier
    classifier=LibLinear(L2R_L2LOSS_SVC)

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #classifier.print_modsel_params()

    # splitting strategy for cross-validation
    splitting_strategy=StratifiedCrossValidationSplitting(labels, 10)

    # evaluation method
    evaluation_criterium=ContingencyTableEvaluation(ACCURACY)

    # cross-validation instance
    cross_validation=CrossValidation(classifier, features, labels,
                                     splitting_strategy, evaluation_criterium)
    cross_validation.set_autolock(False)

    # model selection instance
    model_selection=RandomSearchModelSelection(cross_validation, param_tree_root, 0.5)

    # perform model selection with selected methods
    #print "performing model selection of"
    #param_tree_root.print_tree()
    best_parameters=model_selection.select_model()

    # print best parameters
    #print "best parameters:"
    #best_parameters.print_tree()

    # apply them and print result
    best_parameters.apply_to_machine(classifier)
    result=cross_validation.evaluate()
    #result.print_result()

if __name__=='__main__':
    print('ModelSelectionRandomSearchLibLinear')
    modelselection_random_search_liblinear_modular(*parameter_list[0])



#!/usr/bin/env python

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import sys
import random
from numpy import array
import esvm.parse
import esvm.plots
from esvm.datafuncs import MotifDataDef, fastawrite_sequence, arffwrite_sequence, arffwrite_real
from esvm.mldata import init_datasetfile

if __name__ == '__main__':

    if len(sys.argv)<3 or (sys.argv[1]=='motif' and sys.argv[2]!='arff' and sys.argv[2]!='fasta') \
           or (sys.argv[1]=='motif' and sys.argv[2]=='fasta' and len(sys.argv)<9) \
           or (sys.argv[1]=='motif' and sys.argv[2]=='arff' and len(sys.argv)<14) \
           or (sys.argv[1]=='cloud' and len(sys.argv)<7) or (sys.argv[1]!='motif') \
           and (sys.argv[1]!='cloud'):
        sys.stderr.write( "usage: %s motif fasta MOTIF numSeq seqLenRange"+\
                          "positionRange mutationRate output.fa\n"+\
                          "or: %s motif arff MOTIFPOS numSeq-pos seqLenRange-pos "+\
                          "positionRange-pos mutationRate-pos \\\n"+\
                          "motif-neg numSeq-neg seqLenRange-neg positionRange-neg "+\
                          "mutationRange-neg output.arff\n"+\
                          "or: %s cloud numpoints dimensions fractionOfPositives "+\
                          "cloudWidth output.arff\n" % (sys.argv[0],sys.argv[0],sys.argv[0]) )
        sys.exit(-1)

    random.seed()

    if sys.argv[1] == 'motif':
        if sys.argv[2]=='fasta':
            # generate sequences in FASTA format
            p = MotifDataDef()
            p.motif = sys.argv[3]
            p.numseq = int(sys.argv[4])
            (p.seqlenmin,p.seqlenmax) = esvm.parse.parse_range(sys.argv[5])
            (p.posstart,p.posend) = esvm.parse.parse_range(sys.argv[6])
            p.mutrate = float(sys.argv[7])

            filename = sys.argv[8]
            fastawrite_sequence(filename, p)

        else:
            # generate sequences in ARFF format
            assert(sys.argv[2]=='arff')
            p = MotifDataDef()
            p.motif = sys.argv[3]
            p.numseq = int(sys.argv[4])
            (p.seqlenmin,p.seqlenmax) = esvm.parse.parse_range(sys.argv[5])
            (p.posstart,p.posend) = esvm.parse.parse_range(sys.argv[6])
            p.mutrate = float(sys.argv[7])

            n = MotifDataDef()
            n.motif = sys.argv[8]
            n.numseq = int(sys.argv[9])
            (n.seqlenmin,n.seqlenmax) = esvm.parse.parse_range(sys.argv[10])
            (n.posstart,n.posend) = esvm.parse.parse_range(sys.argv[11])
            n.mutrate = float(sys.argv[12])

            filename = sys.argv[13]
            arffwrite_sequence(filename, p, n)

    elif sys.argv[1] == 'cloud':
        # generate a data cloud in ARFF format
        numpoint = int(sys.argv[2])
        numfeat = int(sys.argv[3])
        fracpos = float(sys.argv[4])
        width = float(sys.argv[5])

        filename = sys.argv[6]
        arffwrite_real(filename, numpoint, numfeat, fracpos, width)
        if len(sys.argv)>=8:
            fp = init_datasetfile(filename,'vec')
            (examples,labels) = fp.readlines()
            pointcloud = []
            for ix in xrange(numpoint):
                pointcloud.append(array([labels[ix],examples[0,ix],examples[1,ix]]))
            esvm.plots.plotcloud(pointcloud,sys.argv[7],'Pointcloud')

	#(examples,labels,metadata)=arffwrite_real(filename, numpoint, numfeat, fracpos, width)
	#if len(sys.argv)>=8:
	#	plots.plotcloud(pointcloud,sys.argv[7],metadata)
    else:
        print 'Unknown option %s\n' % sys.argv[1]



#!/usr/bin/env python
parameter_list = [[1,7],[2,8]]

def classifier_custom_kernel_modular (C=1,dim=7):
	from modshogun import RealFeatures, BinaryLabels, CustomKernel, LibSVM
	from numpy import diag,ones,sign
	from numpy.random import rand,seed

	seed((C,dim))

	lab=sign(2*rand(dim) - 1)
	data=rand(dim, dim)
	symdata=data*data.T + diag(ones(dim))

	kernel=CustomKernel()
	kernel.set_full_kernel_matrix_from_full(data)
	labels=BinaryLabels(lab)
	svm=LibSVM(C, kernel, labels)
	svm.train()
	predictions =svm.apply()
	out=svm.apply().get_labels()
	return svm,out

if __name__=='__main__':
	print('custom_kernel')
	classifier_custom_kernel_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.multiclass_shared import prepare_data

[traindat, label_traindat, testdat, label_testdat] = prepare_data(False)

parameter_list = [[traindat,testdat,label_traindat,label_testdat,2.1,1,1e-5],[traindat,testdat,label_traindat,label_testdat,2.2,1,1e-5]]

def classifier_multiclasslinearmachine_modular (fm_train_real=traindat,fm_test_real=testdat,label_train_multiclass=label_traindat,label_test_multiclass=label_testdat,width=2.1,C=1,epsilon=1e-5):
	from modshogun import RealFeatures, MulticlassLabels
	from modshogun import LibLinear, L2R_L2LOSS_SVC, LinearMulticlassMachine, MulticlassOneVsOneStrategy, MulticlassOneVsRestStrategy

	feats_train = RealFeatures(fm_train_real)
	feats_test  = RealFeatures(fm_test_real)

	labels = MulticlassLabels(label_train_multiclass)

	classifier = LibLinear(L2R_L2LOSS_SVC)
	classifier.set_epsilon(epsilon)
	classifier.set_bias_enabled(True)
	mc_classifier = LinearMulticlassMachine(MulticlassOneVsOneStrategy(), feats_train, classifier, labels)

	mc_classifier.train()
	label_pred = mc_classifier.apply()
	out = label_pred.get_labels()

	if label_test_multiclass is not None:
		from modshogun import MulticlassAccuracy
		labels_test = MulticlassLabels(label_test_multiclass)
		evaluator = MulticlassAccuracy()
		acc = evaluator.evaluate(label_pred, labels_test)
		print('Accuracy = %.4f' % acc)

	return out

if __name__=='__main__':
	print('MulticlassMachine')
	classifier_multiclasslinearmachine_modular(*parameter_list[0])



#!/usr/bin/env python

import numpy
import scipy

from scipy import io
data_dict = scipy.io.loadmat('../data/hmsvm_data_large_integer.mat', struct_as_record=False)

parameter_list=[[data_dict]]

def structure_discrete_hmsvm_bmrm (m_data_dict=data_dict):
	from modshogun import RealMatrixFeatures, SequenceLabels, HMSVMModel, Sequence, TwoStateModel
	from modshogun import StructuredAccuracy, DualLibQPBMSOSVM, SMT_TWO_STATE

	labels_array = m_data_dict['label'][0]

	idxs = numpy.nonzero(labels_array == -1)
	labels_array[idxs] = 0

	labels = SequenceLabels(labels_array, 250, 500, 2)
	features = RealMatrixFeatures(m_data_dict['signal'].astype(float), 250, 500)

	num_obs = 4	# given by the data file used
	model = HMSVMModel(features, labels, SMT_TWO_STATE, num_obs)

	sosvm = DualLibQPBMSOSVM(model, labels, 5000.0)
	sosvm.train()
	#print sosvm.get_w()

	predicted = sosvm.apply(features)
	evaluator = StructuredAccuracy()
	acc = evaluator.evaluate(predicted, labels)
	#print('Accuracy = %.4f' % acc)

if __name__ == '__main__':
	print("Discrete HMSVM BMRM")
	structure_discrete_hmsvm_bmrm(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
trainlabel=lm.load_labels('../data/label_train_dna.dat')
parameter_list=[[traindna,testdna,trainlabel,10,2,True,True,3,0,'n'],
		[traindna,testdna,trainlabel,11,3,True,True,4,0,'n']]

def kernel_polymatchword (fm_train_dna=traindna,fm_test_dna=testdna,
				   label_train_dna=trainlabel,size_cache=10,
				   degree=2,inhomogene=True,normalize=True,
				   order=3,gap=0,reverse='n'):

	sg('add_preproc', 'SORTWORDSTRING')
	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TRAIN')

	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TEST')

	sg('set_kernel', 'POLYMATCH', 'WORD', size_cache, degree, inhomogene, normalize)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('PolyMatchWord')
	kernel_polymatchword(*parameter_list[0])



#!/usr/bin/env python

import numpy as np
from modshogun import TableFactorType

# create the factor type with GT parameters
tid = 0
cards = np.array([2,2], np.int32)
w_gt = np.array([0.3,0.5,1.0,0.2,0.05,0.6,-0.2,0.75])
fac_type = TableFactorType(tid, cards, w_gt)

tid_u = 1
cards_u = np.array([2], np.int32)
w_gt_u = np.array([0.5,0.8,1.0,-0.3])
fac_type_u = TableFactorType(tid_u, cards_u, w_gt_u)

tid_b = 2
cards_b = np.array([2], np.int32)
w_gt_b = np.array([0.8, -0.8])
fac_type_b = TableFactorType(tid_b, cards_b, w_gt_b)

def gen_data(ftype, num_samples, show_data = False):
	from modshogun import Math
	from modshogun import FactorType, Factor, TableFactorType, FactorGraph
	from modshogun import FactorGraphObservation, FactorGraphLabels, FactorGraphFeatures
	from modshogun import MAPInference, TREE_MAX_PROD

	Math.init_random(17)

	samples = FactorGraphFeatures(num_samples)
	labels = FactorGraphLabels(num_samples)

	for i in xrange(num_samples):
		vc = np.array([2,2,2], np.int32)
		fg = FactorGraph(vc)

		data1 = np.array([2.0*Math.random(0.0,1.0)-1.0 for i in xrange(2)])
		vind1 = np.array([0,1], np.int32)
		fac1 = Factor(ftype[0], vind1, data1)
		fg.add_factor(fac1)

		data2 = np.array([2.0*Math.random(0.0,1.0)-1.0 for i in xrange(2)])
		vind2 = np.array([1,2], np.int32)
		fac2 = Factor(ftype[0], vind2, data2)
		fg.add_factor(fac2)

		data3 = np.array([2.0*Math.random(0.0,1.0)-1.0 for i in xrange(2)])
		vind3 = np.array([0], np.int32)
		fac3 = Factor(ftype[1], vind3, data3)
		fg.add_factor(fac3)

		data4 = np.array([2.0*Math.random(0.0,1.0)-1.0 for i in xrange(2)])
		vind4 = np.array([1], np.int32)
		fac4 = Factor(ftype[1], vind4, data4)
		fg.add_factor(fac4)

		data5 = np.array([2.0*Math.random(0.0,1.0)-1.0 for i in xrange(2)])
		vind5 = np.array([2], np.int32)
		fac5 = Factor(ftype[1], vind5, data5)
		fg.add_factor(fac5)

		data6 = np.array([1.0])
		vind6 = np.array([0], np.int32)
		fac6 = Factor(ftype[2], vind6, data6)
		fg.add_factor(fac6)

		data7 = np.array([1.0])
		vind7 = np.array([2], np.int32)
		fac7 = Factor(ftype[2], vind7, data7)
		fg.add_factor(fac7)

		samples.add_sample(fg)
		fg.connect_components()
		fg.compute_energies()

		infer_met = MAPInference(fg, TREE_MAX_PROD)
		infer_met.inference()

		fg_obs = infer_met.get_structured_outputs()
		labels.add_label(fg_obs)

		if show_data:
			state = fg_obs.get_data()
			print state

	return samples, labels


w_all = [w_gt,w_gt_u,w_gt_b]
ftype_all = [fac_type,fac_type_u,fac_type_b]

num_samples = 10
samples, labels = gen_data(ftype_all, num_samples)

parameter_list = [[samples,labels,w_all,ftype_all]]

def structure_factor_graph_model(tr_samples = samples, tr_labels = labels, w = w_all, ftype = ftype_all):
	from modshogun import SOSVMHelper, LabelsFactory
	from modshogun import FactorGraphModel, MAPInference, TREE_MAX_PROD
	from modshogun import DualLibQPBMSOSVM, StochasticSOSVM

	# create model
	model = FactorGraphModel(tr_samples, tr_labels, TREE_MAX_PROD, False)
	w_truth = [w[0].copy(), w[1].copy(), w[2].copy()]
	w[0] = np.zeros(8)
	w[1] = np.zeros(4)
	w[2] = np.zeros(2)
	ftype[0].set_w(w[0])
	ftype[1].set_w(w[1])
	ftype[2].set_w(w[2])
	model.add_factor_type(ftype[0])
	model.add_factor_type(ftype[1])
	model.add_factor_type(ftype[2])

	# --- training with BMRM ---
	bmrm = DualLibQPBMSOSVM(model, tr_labels, 0.01)
	#bmrm.set_verbose(True)
	bmrm.train()
	#print 'learned weights:'
	#print bmrm.get_w()
	#print 'ground truth weights:'
	#print w_truth

	# evaluation
	lbs_bmrm = bmrm.apply()
	acc_loss = 0.0
	ave_loss = 0.0
	for i in xrange(num_samples):
		y_pred = lbs_bmrm.get_label(i)
		y_truth = tr_labels.get_label(i)
		acc_loss = acc_loss + model.delta_loss(y_truth, y_pred)

	ave_loss = acc_loss / num_samples

	#print('BMRM: Average training error is %.4f' % ave_loss)

	# show primal objs and dual objs
	#hbm = bmrm.get_helper()
	#print hbm.get_primal_values()
	#print hbm.get_eff_passes()
	#print hbm.get_train_errors()

	# --- training with SGD ---
	sgd = StochasticSOSVM(model, tr_labels)
	#sgd.set_verbose(True)
	sgd.set_lambda(0.01)
	sgd.train()

	# evaluation
	#print('SGD: Average training error is %.4f' % SOSVMHelper.average_loss(sgd.get_w(), model))
	#hp = sgd.get_helper()
	#print hp.get_primal_values()
	#print hp.get_eff_passes()
	#print hp.get_train_errors()

if __name__ == '__main__':
	print("Factor Graph Model")
	structure_factor_graph_model(*parameter_list[0])



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list = [[traindat,traindat],[traindat,testdat]]

def distance_euclidean_modular(train_fname=traindat,test_fname=testdat):

	from modshogun import RealFeatures, EuclideanDistance, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	distance=EuclideanDistance(feats_train, feats_train)

	dm_train=distance.get_distance_matrix()
	distance.init(feats_train, feats_test)
	dm_test=distance.get_distance_matrix()

	return distance,dm_train,dm_test

if __name__=='__main__':
	print('EuclideanDistance')
	distance_euclidean_modular(*parameter_list[0])



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list = [[traindat,testdat,4,False,True],[traindat,testdat,5,False,True]]

def kernel_poly_modular (train_fname=traindat,test_fname=testdat,degree=4,inhomogene=False,
	use_normalization=True):
	from modshogun import RealFeatures, PolyKernel, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	kernel=PolyKernel(
		feats_train, feats_train, degree, inhomogene, use_normalization)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel
if __name__=='__main__':
	print('Poly')
	kernel_poly_modular (*parameter_list[0])



"""
Test Preprocessor
"""

from sg import sg
import util


def _evaluate (indata, prefix):
	util.set_and_train_kernel(indata)

	kmatrix=sg('get_kernel_matrix', 'TRAIN')
	km_train=max(abs(indata['kernel_matrix_train']-kmatrix).flat)

	kmatrix=sg('get_kernel_matrix', 'TEST')
	km_test=max(abs(indata['kernel_matrix_test']-kmatrix).flat)

	return util.check_accuracy(
		indata[prefix+'accuracy'], km_train=km_train, km_test=km_test)


def _set_preprocessor(indata, prefix):
	pname=util.fix_preproc_name_inconsistency(indata[prefix+'name'])
	args=util.get_args(indata, prefix)

	sg('add_preproc', pname, *args)
	sg('attach_preproc', 'TRAIN')
	sg('attach_preproc', 'TEST')


########################################################################
# public
########################################################################

def test (indata):
	prefix='kernel_'
	try:
		util.set_features(indata, prefix)
	except NotImplementedError, e:
		print e
		return True

	_set_preprocessor(indata, 'preprocessor_')

	return _evaluate(indata, prefix)




#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_twoclass.dat'

parameter_list = [[traindat,testdat,label_traindat,2.1,1]]

def classifier_libsvm_minimal_modular (train_fname=traindat,test_fname=testdat,label_fname=label_traindat,width=2.1,C=1):
	from modshogun import RealFeatures, BinaryLabels
	from modshogun import LibSVM, GaussianKernel, CSVFile
	from modshogun import ErrorRateMeasure

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))
	labels=BinaryLabels(CSVFile(label_fname))
	kernel=GaussianKernel(feats_train, feats_train, width);

	svm=LibSVM(C, kernel, labels);
	svm.train();

	out=svm.apply(feats_train);
	evaluator = ErrorRateMeasure()
	testerr = evaluator.evaluate(out,labels)
	#print(testerr)

if __name__=='__main__':
	print('LibSVM Minimal')
	classifier_libsvm_minimal_modular(*parameter_list[0])



#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2008 Soeren Sonnenburg
# Written (W) 2007 Gunnar Raetsch
# Copyright (C) 2006-2008 Fraunhofer Institute FIRST and Max-Planck-Society
#

import sys
from numpy import mat,array,inf,any,reshape,int32

class model(object):
	#model matrices
	bins=None
	dict_weights_intron=None
	dict_weights_coding=None
	a_trans=None
	p=None
	q=None

	statedescr = None
	plifidmat = None
	orf_info = None
	use_orf = None

	word_degree = None
	mod_words = None
	sign_words = None

	#penalties
	penalty_acceptor_boundaries=None
	penalty_acceptor_penalty=None
	penalty_donor_boundaries=None
	penalty_donor_penalty=None
	penalty_coding_len_boundaries=None
	penalty_coding_len_penalty=None
	penalty_first_coding_len_boundaries=None
	penalty_first_coding_len_penalty=None
	penalty_last_coding_len_boundaries=None
	penalty_last_coding_len_penalty=None
	penalty_single_coding_len_boundaries=None
	penalty_single_coding_len_penalty=None
	penalty_intron_len_boundaries=None
	penalty_intron_len_penalty=None
	penalty_coding_boundaries=None
	penalty_coding_penalty=None
	penalty_coding2_boundaries=None
	penalty_coding2_penalty=None
	penalty_coding3_boundaries=None
	penalty_coding3_penalty=None
	penalty_coding4_boundaries=None
	penalty_coding4_penalty=None
	penalty_intron_boundaries=None
	penalty_intron_penalty=None
	penalty_intron2_boundaries=None
	penalty_intron2_penalty=None
	penalty_intron3_boundaries=None
	penalty_intron3_penalty=None
	penalty_intron4_boundaries=None
	penalty_intron4_penalty=None
	penalty_transitions_penalty=None

	#acceptor
	acc_splice_b=None
	acc_splice_order=None
	acc_splice_window_left=None
	acc_splice_window_right=None
	acc_splice_alphas=None
	acc_splice_svs=None

	#donor
	don_splice_b=None
	don_splice_order=None
	don_splice_use_gc=None
	don_splice_window_left=None
	don_splice_window_right=None
	don_splice_alphas=None
	don_splice_svs=None



def parse_file(file):
	m=model()

	l=file.readline();

	if l != '%msplicer definition file version: 1.0\n':
		sys.stderr.write("\nfile not a msplicer definition file\n")
		return None

	while l:
		if not ( l.startswith('%') or l.startswith('\n') ): # comment
			if m.bins is None: m.bins=parse_value(l, 'bins')
			if m.dict_weights_intron is None: m.dict_weights_intron=parse_matrix(l, file, 'dict_weights_intron')
			if m.dict_weights_coding is None: m.dict_weights_coding=parse_matrix(l, file, 'dict_weights_coding')
			if m.a_trans is None: m.a_trans=parse_matrix(l, file, 'msplicer_a_trans')
			if m.p is None:
				m.p=parse_vector(l, file, 'msplicer_p')
				if m.p is not None:
					m.p[m.p==32768]=-inf
			if m.q is None:
				m.q=parse_vector(l, file, 'msplicer_q')
				if m.q is not None:
					m.q[m.q==32768]=-inf

			if m.statedescr is None:
				m.statedescr=parse_vector(l, file, 'statedescr')
				if m.statedescr is not None:
					m.statedescr=array(m.statedescr, int32)

			if m.plifidmat is None:
				m.plifidmat=parse_matrix(l, file, 'plifidmat')
				if m.plifidmat is not None:
					m.plifidmat = array(m.plifidmat, int32)

			if m.orf_info is None:
				m.orf_info=parse_matrix(l, file, 'orf_info')
				if m.orf_info is not None:
					m.orf_info=array(m.orf_info, int32).T
					if any(m.orf_info != -1):
						m.use_orf = True
					else:
						m.use_orf = False

			if m.word_degree is None: m.word_degree=parse_vector(l, file, 'word_degree')
			if m.mod_words is None: m.mod_words=parse_matrix(l, file, 'mod_words')
			if m.sign_words is None: m.sign_words=parse_vector(l, file, 'sign_words')

			#penalties
			if m.penalty_acceptor_boundaries is None: m.penalty_acceptor_boundaries=parse_vector(l, file, 'penalty_acceptor_boundaries')
			if m.penalty_acceptor_penalty is None: m.penalty_acceptor_penalty=parse_vector(l, file, 'penalty_acceptor_penalty')
			if m.penalty_donor_boundaries is None: m.penalty_donor_boundaries=parse_vector(l, file, 'penalty_donor_boundaries')
			if m.penalty_donor_penalty is None: m.penalty_donor_penalty=parse_vector(l, file, 'penalty_donor_penalty')
			if m.penalty_coding_len_boundaries is None: m.penalty_coding_len_boundaries=parse_vector(l, file, 'penalty_coding_len_boundaries')
			if m.penalty_coding_len_penalty is None: m.penalty_coding_len_penalty=parse_vector(l, file, 'penalty_coding_len_penalty')
			if m.penalty_first_coding_len_boundaries is None: m.penalty_first_coding_len_boundaries=parse_vector(l, file, 'penalty_first_coding_len_boundaries')
			if m.penalty_first_coding_len_penalty is None: m.penalty_first_coding_len_penalty=parse_vector(l, file, 'penalty_first_coding_len_penalty')
			if m.penalty_last_coding_len_boundaries is None: m.penalty_last_coding_len_boundaries=parse_vector(l, file, 'penalty_last_coding_len_boundaries')
			if m.penalty_last_coding_len_penalty is None: m.penalty_last_coding_len_penalty=parse_vector(l, file, 'penalty_last_coding_len_penalty')
			if m.penalty_single_coding_len_boundaries is None: m.penalty_single_coding_len_boundaries=parse_vector(l, file, 'penalty_single_coding_len_boundaries')
			if m.penalty_single_coding_len_penalty is None: m.penalty_single_coding_len_penalty=parse_vector(l, file, 'penalty_single_coding_len_penalty')
			if m.penalty_intron_len_boundaries is None: m.penalty_intron_len_boundaries=parse_vector(l, file, 'penalty_intron_len_boundaries')
			if m.penalty_intron_len_penalty is None: m.penalty_intron_len_penalty=parse_vector(l, file, 'penalty_intron_len_penalty')
			if m.penalty_coding_boundaries is None: m.penalty_coding_boundaries=parse_vector(l, file, 'penalty_coding_boundaries')
			if m.penalty_coding_penalty is None: m.penalty_coding_penalty=parse_vector(l, file, 'penalty_coding_penalty')
			if m.penalty_coding2_boundaries is None: m.penalty_coding2_boundaries=parse_vector(l, file, 'penalty_coding2_boundaries')
			if m.penalty_coding2_penalty is None: m.penalty_coding2_penalty=parse_vector(l, file, 'penalty_coding2_penalty')
			if m.penalty_coding3_boundaries is None: m.penalty_coding3_boundaries=parse_vector(l, file, 'penalty_coding3_boundaries')
			if m.penalty_coding3_penalty is None: m.penalty_coding3_penalty=parse_vector(l, file, 'penalty_coding3_penalty')
			if m.penalty_coding4_boundaries is None: m.penalty_coding4_boundaries=parse_vector(l, file, 'penalty_coding4_boundaries')
			if m.penalty_coding4_penalty is None: m.penalty_coding4_penalty=parse_vector(l, file, 'penalty_coding4_penalty')
			if m.penalty_intron_boundaries is None: m.penalty_intron_boundaries=parse_vector(l, file, 'penalty_intron_boundaries')
			if m.penalty_intron_penalty is None: m.penalty_intron_penalty=parse_vector(l, file, 'penalty_intron_penalty')
			if m.penalty_intron2_boundaries is None: m.penalty_intron2_boundaries=parse_vector(l, file, 'penalty_intron2_boundaries')
			if m.penalty_intron2_penalty is None: m.penalty_intron2_penalty=parse_vector(l, file, 'penalty_intron2_penalty')
			if m.penalty_intron3_boundaries is None: m.penalty_intron3_boundaries=parse_vector(l, file, 'penalty_intron3_boundaries')
			if m.penalty_intron3_penalty is None: m.penalty_intron3_penalty=parse_vector(l, file, 'penalty_intron3_penalty')
			if m.penalty_intron4_boundaries is None: m.penalty_intron4_boundaries=parse_vector(l, file, 'penalty_intron4_boundaries')
			if m.penalty_intron4_penalty is None: m.penalty_intron4_penalty=parse_vector(l, file, 'penalty_intron4_penalty')
			if m.penalty_transitions_penalty is None: m.penalty_transitions_penalty=parse_vector(l, file, 'penalty_transitions_penalty')

			#acceptor
			if m.acc_splice_b is None: m.acc_splice_b=parse_value(l, 'acc_splice_b')
			if m.acc_splice_order is None: m.acc_splice_order=parse_value(l, 'acc_splice_order')
			if m.acc_splice_window_left is None: m.acc_splice_window_left=parse_value(l, 'acc_splice_window_left')
			if m.acc_splice_window_right is None: m.acc_splice_window_right=parse_value(l, 'acc_splice_window_right')
			if m.acc_splice_alphas is None: m.acc_splice_alphas=parse_vector(l, file, 'acc_splice_alphas')
			if m.acc_splice_svs is None: m.acc_splice_svs=parse_string(l, file, 'acc_splice_svs')

			#donor
			if m.don_splice_b is None: m.don_splice_b=parse_value(l, 'don_splice_b')
			if m.don_splice_order is None: m.don_splice_order=parse_value(l, 'don_splice_order')
			if m.don_splice_use_gc is None: m.don_splice_use_gc=parse_value(l, 'don_splice_use_gc')
			if m.don_splice_window_left is None: m.don_splice_window_left=parse_value(l, 'don_splice_window_left')
			if m.don_splice_window_right is None: m.don_splice_window_right=parse_value(l, 'don_splice_window_right')
			if m.don_splice_alphas is None: m.don_splice_alphas=parse_vector(l, file, 'don_splice_alphas')
			if m.don_splice_svs is None: m.don_splice_svs=parse_string(l, file, 'don_splice_svs')

		l=file.readline()

	sys.stderr.write('done\n')
	return m

def parse_value(line, name):
	if (line.startswith(name)):
		sys.stdout.write('.'); sys.stdout.flush()
		return float(line[line.find('=')+1:-1])
	else:
		return None

def parse_vector(line, file, name):
    mat = parse_matrix(line, file, name)
    if mat is None:
     return mat
    else:
     mat = array(mat).flatten()
     return mat

def parse_matrix(line, file, name):
	if (line.startswith(name)):
		sys.stdout.write('.'); sys.stdout.flush()
		if line.find(']') < 0:
			l=''
			while l is not None and l.find(']') < 0:
				line+=l
				l=file.readline()
			if l is not None and l.find(']') >= 0:
				line+=l

		if line.find(']') < 0:
			sys.stderr.write("matrix `" + name + "' ended without ']'\n")
			return None
		else:
			mm = mat(line[line.find('['):line.find(']')+1])
			if len(mm.shape)==1:
				mm = reshape(mm.shape[0],1)
			return mm
	else:
		return None

def parse_string(line, file, name):
	if (line.startswith(name)):
		sys.stdout.write('.'); sys.stdout.flush()
		l=''
		lines=[]
		while l is not None and l.find(']') < 0:
			if l:
				lines.append(l[:-1])
			l=file.readline()

		if l.find(']') < 0:
			sys.stderr.write("string ended without ']'\n")
			return None
		else:
			return lines
	else:
		return None

if __name__ == '__main__':
	import bz2
	import sys
	import hotshot, hotshot.stats

	def load():
		#f=bz2.BZ2File('data/msplicer_arabidopsis10_gc=1_orf=0.dat.bz2');
		f=file('data/msplicer_arabidopsis10_gc=1_orf=0.dat');
		m=parse_file(f);

		print m.penalty_acceptor_boundaries is None
		print m.penalty_acceptor_penalty is None
		print m.penalty_donor_boundaries is None
		print m.penalty_donor_penalty is None
		print m.penalty_coding_len_boundaries is None
		print m.penalty_coding_len_penalty is None
		print m.penalty_first_coding_len_boundaries is None
		print m.penalty_first_coding_len_penalty is None
		print m.penalty_last_coding_len_boundaries is None
		print m.penalty_last_coding_len_penalty is None
		print m.penalty_single_coding_len_boundaries is None
		print m.penalty_single_coding_len_penalty is None
		print m.penalty_intron_len_boundaries is None
		print m.penalty_intron_len_penalty is None
		print m.penalty_coding_boundaries is None
		print m.penalty_coding_penalty is None
		print m.penalty_coding2_boundaries is None
		print m.penalty_coding2_penalty is None
		print m.penalty_coding3_boundaries is None
		print m.penalty_coding3_penalty is None
		print m.penalty_coding4_boundaries is None
		print m.penalty_coding4_penalty is None
		print m.penalty_intron_boundaries is None
		print m.penalty_intron_penalty is None
		print m.penalty_intron2_boundaries is None
		print m.penalty_intron2_penalty is None
		print m.penalty_intron3_boundaries is None
		print m.penalty_intron3_penalty is None
		print m.penalty_intron4_boundaries is None
		print m.penalty_intron4_penalty is None
		print m.penalty_transitions_penalty is None

		print m.acc_splice_b is None
		print m.acc_splice_order is None
		print m.acc_splice_window_left is None
		print m.acc_splice_window_right is None
		print m.acc_splice_alphas is None
		print m.acc_splice_svs is None

		print m.don_splice_b is None
		print m.don_splice_order is None
		print m.don_splice_use_gc is None
		print m.don_splice_window_left is None
		print m.don_splice_window_right is None
		print m.don_splice_alphas is None
		print m.don_splice_svs is None

	load()

	#prof = hotshot.Profile("model.prof")
	#benchtime = prof.runcall(load)
	#prof.close()
	#stats = hotshot.stats.load("model.prof")
	#stats.strip_dirs()
	#stats.sort_stats('time', 'calls')
	#stats.print_stats(20)



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindna = lm.load_dna('../data/fm_train_dna.dat')

parameter_list = [[traindna,3,0,False],[traindna,4,0,False]]

def distribution_histogram_modular (fm_dna=traindna,order=3,gap=0,reverse=False):
	from modshogun import StringWordFeatures, StringCharFeatures, DNA
	from modshogun import Histogram

	charfeat=StringCharFeatures(DNA)
	charfeat.set_features(fm_dna)
	feats=StringWordFeatures(charfeat.get_alphabet())
	feats.obtain_from_char(charfeat, order-1, order, gap, reverse)

	histo=Histogram(feats)
	histo.train()

	histo.get_histogram()

	num_examples=feats.get_num_vectors()
	num_param=histo.get_num_model_parameters()
	#for i in xrange(num_examples):
	#	for j in xrange(num_param):
	#		histo.get_log_derivative(j, i)

	out_likelihood = histo.get_log_likelihood()
	out_sample = histo.get_log_likelihood_sample()
	return histo,out_sample,out_likelihood
###########################################################################
# call functions
###########################################################################

if __name__=='__main__':
	print('Histogram')
	distribution_histogram_modular(*parameter_list[0])

